/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 *
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_ERR

#include <stdlib.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h>

#include <owl/core/owl_defs.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_os.h>
#include <owl/core/owl_err.h>
#include <owl/os/owl_mem.h>

#if OWL_CONFIG_THREAD
# include <owl/core/owl_thread.h>
static owlmutex_t lock;
#endif /* OWL_CONFIG_THREAD */

static bool_t initialized = OWL_FALSE;

int
owl_os_init(void)
{
#if OWL_CONFIG_THREAD
	if (owlmutex_create(&lock) < 0)
                return OWL_FAILURE;
#endif /* OWL_CONFIG_THREAD */

        owl_mem_init();
	initialized = OWL_TRUE;

	return 0;
}

void *
owl_os_memcpy(void *dst, const void *src, size_t n)
{
        return memcpy(dst, src, n);
}

void *
owl_os_memset(void *s, int c, size_t n)
{
        return memset(s, c, n);
}

void *
owl_os_memmove(void *dst, const void *src, size_t n)
{
        return memmove(dst, src, n);
}

size_t
owl_os_strlen(const char *s)
{
        return strlen(s);
}

char *
owl_os_strncpy(char *dst, const char *src, size_t n)
{
        return strncpy(dst, src, n);
}

int
owl_os_strncmp(const char *s1, const char *s2, size_t n)
{
        return strncmp(s1, s2, n);
}

int
owl_os_strcmp(const char *s1, const char *s2)
{
        return strcmp(s1, s2);
}

char *
owl_os_strcpy(char *dst, const char *src)
{
        return strcpy(dst, src);
}

char *
owl_mem_strdup(const char *s)
{
        size_t l = owl_os_strlen(s);
        return owl_os_strndup(s, l);
}

char *
owl_mem_strndup(const char *s, size_t n)
{
        char *s2 = owl_os_alloc(n + 1);
        if (s2 == NULL)
                return NULL;

        owl_os_strncpy(s2, s, n);
        s2[n] = 0;
        return s2;
}

char *
owl_os_strcat(char *dst, const char *src)
{
        return strcat(dst, src);
}

int
owl_os_memcmp(const void *s1, const void *s2, size_t n)
{
        return memcmp(s1, s2, n);
}

long int
owl_os_strtol(const char *nptr, char **endptr, int base)
{
        return strtol(nptr, endptr, base);
}

char *
owl_os_strchr(const char *s, int c)
{
        return strchr(s, c);
}

char *
owl_os_strrchr(const char *s, int c)
{
        return strrchr(s, c);
}

int
owl_os_strcasecmp(const char *s1, const char *s2)
{
        return strcasecmp(s1, s2);
}

char *
owl_os_strstr(const char *haystack, const char *needle)
{
        return strstr(haystack, needle);
}

char *
owl_os_strtok(char *s, const char *delim)
{
        return strtok(s, delim);
}

int
owl_os_strncasecmp(const char *s1, const char *s2, size_t n)
{
        return strncasecmp(s1, s2, n);
}

char *
owl_os_strcasestr(const char *haystack, const char *needle)
{
        size_t hay_len = strlen(haystack);
        size_t needle_len = strlen(needle);
        while (hay_len >= needle_len) {
                if (owl_os_strncasecmp(haystack, needle, needle_len) == 0)
                        return (char *) haystack;

                haystack++;
                hay_len--;
        }

        return NULL;
}

int
owl_os_snprintf(char *str, size_t size, const char *format, ...)
{
        int ret;
        va_list ap;
        va_start(ap, format);
        ret = owl_os_vsnprintf(str, size, format, ap);
        va_end(ap);
        return ret;
}

int
owl_os_vsnprintf(char *str, size_t size, const char *format, va_list ap)
{
        return vsniprintf(str, size, format, ap);
}

int
owl_os_printf(const char *fmt, ...)
{
#if OWL_CONFIG_OS_PRINTF_STACK
        char str[OWL_CONFIG_OS_PRINTF_LEN];
#else
        char *str = NULL;
#endif /* OWL_CONFIG_OS_PRINTF_STATIC */
        va_list args;
        int len;
        const char *iter;

	if (!initialized)
                return 0;

#if OWL_CONFIG_THREAD
	owlmutex_lock(&lock);
#endif /* OWL_CONFIG_THREAD */

        va_start(args, fmt);

#if !OWL_CONFIG_OS_PRINTF_STACK
        if ((str = owl_os_alloc(OWL_CONFIG_OS_PRINTF_LEN)) == NULL)
                goto err;
#endif /* !OWL_CONFIG_OS_PRINTF_STATIC */

        if ((len = owl_os_vsnprintf(
                     str, OWL_CONFIG_OS_PRINTF_LEN, fmt, args)) < 0) {
#if !OWL_CONFIG_OS_PRINTF_STACK
                owl_os_free(str);
#endif /* !OWL_CONFIG_OS_PRINTF_STATIC */
                goto err;
        }

        iter = str;
        while (*iter) {

#if OWL_CONFIG_OS_CRLF
		if (*iter == '\n')
			owl_putc('\r');
#endif /* OWL_CONFIG_OS_CRLF */

		owl_putc(*iter++);
        }

#if !OWL_CONFIG_OS_PRINTF_STACK
        owl_os_free(str);
#endif /* !OWL_CONFIG_OS_PRINTF_STATIC */

#if OWL_CONFIG_THREAD
	owlmutex_unlock(&lock);
#endif /* OWL_CONFIG_THREAD */

        return len;

err:
        /* If we don't have memory, then print the fmt string */
        iter = fmt;
        while (*iter)
                owl_putc(*iter++);

#if OWL_CONFIG_THREAD
	owlmutex_unlock(&lock);
#endif /* OWL_CONFIG_THREAD */

        return -1;
}
