#  Copyright (C) 2012, H&D Wireless AB All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  1. Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#
#  2. Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#
#  3. The name of H&D Wireless AB may not be used to endorse or promote
#     products derived from this software without specific prior written
#     permission.
#
#  4. This software may only be redistributed and used in connection with an
#     H&D Wireless product.
#
#  THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
#  SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
#  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

APP_PATH ?= .
TRUNK_PATH = $(APP_PATH)/../../../..
PLATFORM_PATH = $(TRUNK_PATH)/platform/linux
TOOLKIT_PATH = $(TRUNK_PATH)/toolkit

BOARD ?= desktop
CONFIG ?= stack-offload
OS = libc
DEBUG ?= 0
BUILD_RELEASE = "2.3.0"

include $(PLATFORM_PATH)/board/$(BOARD)/board.mk
include $(TRUNK_PATH)/arch/$(ARCH)/gcc/arch.mk
include $(PLATFORM_PATH)/owl/owl.mk
include $(TRUNK_PATH)/toolkit/lwip/lwip.mk
MK_FILES += $(PLATFORM_PATH)/board/$(BOARD)/board.mk
MK_FILES += $(TRUNK_PATH)/arch/$(ARCH)/gcc/arch.mk
MK_FILES += $(PLATFORM_PATH)/owl/owl.mk
MK_FILES += $(TRUNK_PATH)/toolkit/lwip/lwip.mk

# Output name
ifeq (1,$(DEBUG))
 TARGET_DIR = pico_client-$(CONFIG)-$(BOARD)-debug
else
 TARGET_DIR = pico_client-$(CONFIG)-$(BOARD)
endif
TARGET = $(TARGET_DIR).elf

# Include path
INC_PATH += $(BOARD_INCS)
INC_PATH += $(ARCH_INCS)

# Libraries
LIBS += $(BOARD_LIBS)
LDFLAGS += $(BOARD_LDFLAGS)
LDFLAGS += $(ARCH_LDFLAGS)
LDF_EXTRA_FLAGS += $(ARCH_LD_EXTRA_FLAGS)

# Defines
DEFS += $(BOARD_DEFS)
DEFS += $(ARCH_DEFS)
DEFS += -DOWL_CONFIG_MTD=0
DEFS += -DOWL_CONFIG_SHELL_PRINTF_LEN=1024
DEFS += -DWLP_CONFIG_HPI=WLP_CONFIG_HPI_UART

CFLAGS += $(BOARD_CFLAGS)
CFLAGS += $(ARCH_CFLAGS)

# C source files
CSRCS += $(BOARD_SRCS)
CSRCS += $(TRUNK_PATH)/wlp_api/hdg205/wlp_api.c
CSRCS += $(TRUNK_PATH)/wlp_api/hdg205/wlp_firmware.c
CSRCS += $(TRUNK_PATH)/wlp_api/hdg205/wlp_uart.c
CSRCS += $(TRUNK_PATH)/wlp_api/hdg205/wlp_spi.c
CSRCS += $(TRUNK_PATH)/wlp_api/hdg205/wlp_sdio.c
CSRCS += $(TOOLKIT_PATH)/owl/core/owl_shell.c
CSRCS += $(TOOLKIT_PATH)/owl/core/owl_debug.c
CSRCS += $(TOOLKIT_PATH)/owl/core/owl_err.c
CSRCS += $(TOOLKIT_PATH)/owl/core/owl_timer.c
CSRCS += $(TOOLKIT_PATH)/owl/components/hdg205.c
CSRCS += $(TOOLKIT_PATH)/owl/util/cbuf.c
CSRCS += $(TOOLKIT_PATH)/pico/shell/mgmt.c
CSRCS += $(TOOLKIT_PATH)/pico/shell/link.c
CSRCS += $(TOOLKIT_PATH)/pico/shell/ipaddr.c
CSRCS += $(TOOLKIT_PATH)/pico/shell/loopback.c
CSRCS += $(TOOLKIT_PATH)/pico/shell/rftest.c
CSRCS += $(TOOLKIT_PATH)/pico/shell/mcast.c
CSRCS += $(TOOLKIT_PATH)/pico/wlp_apitest.c
CSRCS += $(PLATFORM_PATH)/owl/owl_uart.c
CSRCS += $(PLATFORM_PATH)/owl/owl_led.c
CSRCS += $(PLATFORM_PATH)/owl/owl_board.c
CSRCS += $(APP_PATH)/main.c

ifeq (stack-host,$(CONFIG))
 INC_PATH += $(LWIP_INCS)

 DEFS += $(LWIP_DEFS)
 DEFS += -DWLP_CONFIG_STACK_OFFLOAD=0
 DEFS += -DWLP_CONFIG_LWIP=1

 CFLAGS += $(LWIP_CFLAGS)

 CSRCS += $(TOOLKIT_PATH)/owl/core/owl_lwip.c
 CSRCS += $(TOOLKIT_PATH)/owl/util/ttcp.c
 CSRCS += $(LWIP_SRCS)
 CSRCS += $(TOOLKIT_PATH)/pico/lwip/ipaddr.c
else ifeq (stack-offload,$(CONFIG))
 CSRCS += $(TOOLKIT_PATH)/pico/wlp_ttcp.c
 CSRCS += $(TOOLKIT_PATH)/pico/wlp_ntp.c
 CSRCS += $(TOOLKIT_PATH)/pico/wlp_ping.c
 CSRCS += $(TOOLKIT_PATH)/pico/httpd/httpd.c
 CSRCS += $(TOOLKIT_PATH)/pico/httpd/httpd_404.c
 CSRCS += $(TOOLKIT_PATH)/pico/shell/socket.c
 CSRCS += $(TOOLKIT_PATH)/pico/shell/service.c
 CSRCS += $(TRUNK_PATH)/wlp_api/wlp_inet.c
 CSRCS += $(APP_PATH)/http.c
 CSRCS += $(APP_PATH)/upgrade.c

 DEFS += -DWLP_CONFIG_STACK_OFFLOAD=1
else
 $(error "invalid config")
endif
