MAKE      = make
TGTTYPE   = $(suffix $(TARGET))

RM        = rm -Rf

AR        = $(CROSS_COMPILE)ar
ARFLAGS   += rcs

CPP       = $(CC) -E
CPPFLAGS  += $(WARNINGS) \
            $(PLATFORM_INC_PATH:%=-I%) $(INC_PATH:%=-I%) $(CPP_EXTRA_FLAGS) \
            -iquote..

CC        = $(CROSS_COMPILE)gcc
CFLAGS    += $(OPTIMIZATION) $(C_EXTRA_FLAGS) \
            $(PLATFORM_INC_PATH:%=-Wa,-I%) $(AS_EXTRA_FLAGS) \
            -iquote..

AS        = $(CROSS_COMPILE)as
ASFLAGS   = $(OPTIMIZATION) \
	    $(WARNINGS) \
            $(PLATFORM_INC_PATH:%=-Wa,-I%) $(AS_EXTRA_FLAGS)

LD        = $(CROSS_COMPILE)gcc
LDFLAGS   += $(OPTIMIZATION) \
            $(LINKER_SCRIPT:%=-T%)
LOADLIBES +=
LDLIBS    += $(LIBS:%=-l%)

OBJDUMP   = $(CROSS_COMPILE)objdump

NM        = $(CROSS_COMPILE)nm

OBJCOPY   = $(CROSS_COMPILE)objcopy
LSS       = $(TARGET:$(TGTTYPE)=.lss)
HEX       = $(TARGET:$(TGTTYPE)=.hex)
BIN       = $(TARGET:$(TGTTYPE)=.bin)
SYM       = $(TARGET:$(TGTTYPE)=.sym)
OBJ       = $(TARGET:$(TGTTYPE)=.o)
LIB       = lib$(TARGET:$(TGTTYPE)=.a)
IMG       = $(TARGET:$(TGTTYPE)=.img)
ARX       = $(TARGET:$(TGTTYPE)=.arx)

SIZE      = $(CROSS_COMPILE)size

DOCGEN    = doxygen

RELEASE_CMD = $(TRUNK_PATH)/scripts/mk_release.pl

# Create extended listing from target output file.
.PHONY: lss
lss: $(LSS)

# Assembly listing
$(LSS): $(TARGET)
	$(OBJDUMP) -h -S $< > $@

# Create symbol table from target output file.
.PHONY: sym
sym: $(SYM)

# Create symbol table from target output file.
$(SYM): $(TARGET)
	$(NM) -n $< > $@

# Create binary image from ELF output file.
.PHONY: bin
bin: $(BIN)

# Create binary image from ELF output file.
$(BIN): $(TARGET)
	$(OBJCOPY) -O binary $< $@


# NOTE: The 'sizes' target has a lot of unnessecary deps, this is to 
#	ensure that this will be performed as the last step of the build. 
#	Otherwise the sizes printout might be interleaved by some other build
#	steps.
.PHONY: sizes
sizes: $(OBJFILES) $(TARGET)
	@$(SIZE) $(OBJFILES) $(TARGET)
ifeq ($(TGTTYPE),.elf)
	@$(SIZE) -Ax $<
	@$(SIZE) -Bx $<
endif

tags : $(TARGET)
	rm -f ../TAGS
	cat $(DEPFILES) | sort | uniq | sed 's/.*:.*$$//' | tr -d '\\' | tr "\n" " " | xargs -n 1 readlink -f | sort | uniq | xargs etags -a 
	cp TAGS ..

# Find library files we link with. We look for the files in LIBS in
# all the dirs specified in LIB_PATH.
LIB_FILES = $(realpath $(foreach dir,$(LIB_PATH),$(addprefix $(dir)/lib,$(addsuffix .a,$(LIBS)))))
.PHONY: release
release: $(TARGET)
	$(shell echo PATH $(realpath $(TRUNK_PATH)) > release.cfg)
	$(shell echo PATH $(realpath $(EXT_PATH)) >> release.cfg)
	$(shell echo MK_FILES $(realpath $(MK_FILES)) >> release.cfg)
	$(shell echo $(realpath $(RELEASE_EXTRA_FILES)) >> release.cfg)
	$(shell echo $(realpath $(LINKER_SCRIPT)) >> release.cfg)
	$(shell echo $(LIB_FILES) >> release.cfg)
	$(shell cat $(DEPFILES) | sort | uniq | sed 's/.*:\(.*$$\)/\1/' | tr -d '\\' | tr "\n" " " | xargs -n 1 readlink -f | sort | uniq >> release.cfg)
	$(RELEASE_CMD) -n $(basename $(TARGET)) release.cfg

gdb: $(TARGET)
	gdb $(TARGET)

all: lss sym bin sizes
