/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef BOARD_H
#define BOARD_H

#include <stm32f4xx.h>
#include <hdg205/wlp_hdg205.h>

#define OWL_BOARD_SYSTICK_HZ           500
#define OWL_BOARD_SYSTICK_PERIOD       (1000 / OWL_BOARD_SYSTICK_HZ)
#define STM32_BOARD_LSI_FREQ           LSI_VALUE
#define STM32_BOARD_FAMILY             STM32_FAMILY_F4XX

#define OWL_BOARD_SHELL_UART           USART3
#define OWL_BOARD_SHELL_UART_TX_PIN    STM32_CREATE_PIN(GPIOB, 10);
#define OWL_BOARD_SHELL_UART_RX_PIN    STM32_CREATE_PIN(GPIOB, 11);

#if defined(OWL_BOARD_SHELL_UART)
#define OWL_BOARD_DEBUG_UART           OWL_BOARD_SHELL_UART
#endif

#if WLP_CONFIG_HPI == WLP_CONFIG_HPI_SPI

#define OWL_BOARD_HDG205_SPI              SPI3
#define OWL_BOARD_HDG205_SPI_FREQ         2000000
#define OWL_BOARD_HDG205_SPI_CS_PIN       STM32_CREATE_PIN(GPIOD, 1)
#define OWL_BOARD_HDG205_SPI_MISO_PIN     STM32_CREATE_PIN(GPIOC, 11)
#define OWL_BOARD_HDG205_SPI_MOSI_PIN     STM32_CREATE_PIN(GPIOC, 12)
#define OWL_BOARD_HDG205_SPI_SCL_PIN      STM32_CREATE_PIN(GPIOC, 10)
#define OWL_BOARD_HDG205_SPI_IRQ_PIN      STM32_CREATE_PIN(GPIOC, 9)
#define OWL_BOARD_HDG205_SPI_CS           STM32_CREATE_PIN(GPIOD, 1)

#elif WLP_CONFIG_HPI == WLP_CONFIG_HPI_SDIO

#define OWL_BOARD_HDG205_SDIO             NULL
#define OWL_BOARD_HDG205_SDIO_CMD_PIN     STM32_CREATE_PIN(GPIOD, 2)
#define OWL_BOARD_HDG205_SDIO_SCK_PIN     STM32_CREATE_PIN(GPIOC, 12)
#define OWL_BOARD_HDG205_SDIO_D0_PIN      STM32_CREATE_PIN(GPIOC, 8)
#define OWL_BOARD_HDG205_SDIO_D1_PIN      STM32_CREATE_PIN(GPIOC, 9)
#define OWL_BOARD_HDG205_SDIO_D2_PIN      STM32_CREATE_PIN(GPIOC, 10)
#define OWL_BOARD_HDG205_SDIO_D3_PIN      STM32_CREATE_PIN(GPIOC, 11)
#define OWL_BOARD_HDG205_SDIO_DETECT_PIN  STM32_CREATE_PIN(GPIOH, 13)

#endif /* WLP_CONFIG_HPI */

#define OWL_BOARD_LED1                    STM32_CREATE_PIN(GPIOG, 6)
#define OWL_BOARD_LED2                    STM32_CREATE_PIN(GPIOG, 8)
#define OWL_BOARD_LED3                    STM32_CREATE_PIN(GPIOI, 9)
#define OWL_BOARD_LED4                    STM32_CREATE_PIN(GPIOC, 7)

#define OWL_BOARD_BUTTON0                 STM32_CREATE_PIN(GPIOG, 15)

extern const struct gpiomap gpiomap[49];

#endif /* BOARD_H */
