# Copyright (C) 2012, H&D Wireless AB All rights reserved.
#
# The license to use this software in whole and in part and to
# redistribute it in any form follows with the WiFi HW module from H&D
# Wireless and is granted under the following restrictions:
#
# 1. Redistributions of source code must retain the above copyright
#    notice, this list of conditions and the following disclaimer.
#
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions and the following disclaimer in the
#    documentation and/or other materials provided with the distribution.
#
# 3. The name of H&D Wireless AB may not be used to endorse or promote
#    products derived from this software without specific prior written
#    permission.
#
# 4. The software may only be used together with hardware from H&D
#    Wireless all other use is prohibited.
#
# 5. The license to use and redistribute the software is granted
#    together with the purchase of a hardware platform on a one to one
#    basis.
#
# 6. The binary code may not be reversed engineered or by other means
#    copied to circumvent this license.
#
# THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
# EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
# IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
# SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
# INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
# (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
# SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
# STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
# ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
# OF THE POSSIBILITY OF SUCH DAMAGE.
#
# For more information regarding this software license Contact H&D
# Wireless AB (support@hd-wireless.se).

FRAMEWORK_PATH ?= $(EXT_PATH)/platform/stm32/STM32F4xx_DSP_StdPeriph_Lib_V1.2.0

DRIVER_PATH = $(FRAMEWORK_PATH)/Libraries/STM32F4xx_StdPeriph_Driver

ARCH = cm4

LINKER_SCRIPT = $(PLATFORM_PATH)/board/$(BOARD)/stm32.ld

OPENOCD_TARGET = stm32f4x
OPENOCD_LINK = rlink

BOARD_LIBS += 
LIB_PATH += $(TRUNK_PATH)/wl_api/build/gcc

UC_BOARD := $(shell echo $(BOARD) | tr '[:lower:]' '[:upper:]' | tr '-' '_')

BOARD_DEFS  = -DBOARD=$(UC_BOARD) -DUSE_STDPERIPH_DRIVER
BOARD_DEFS += -DSYSCLK_FREQ_168MHz=168000000 -DSTM32F40XX
BOARD_DEFS += -DUSE_STM324xG_EVAL -DHSE_VALUE=25000000
BOARD_DEFS += -DBYTE_ORDER=LITTLE_ENDIAN

BOARD_LDFLAGS += --gc-sections

# The smallest erase size of the external flash. This is required since during
# upgrade, each description will be processed by first erasing the area that 
# is occupied by the data.
EXTFLASH_ERASESIZE = 4096

# Edit this and rebuild bootloader and application to change the size of
# the bootloader.
#
BOOTLOADER_SIZE = 8192

# The internal FLASH mem size, 1MByte
INTFLASH_SIZE = 0x100000

# Internal Flash mtd size 128 KBytes, Set to 0 if not used
BOARD_DEFS += -DINTFLASH_MTD_SIZE=0x20000

# Application size, aligned to EXTFLASH_ERASESIZE
APPLICATION_SIZE := $(shell echo " \
 ($(APPLICATION_SIZE) - 1) / $(EXTFLASH_ERASESIZE) * \
 $(EXTFLASH_ERASESIZE) + \
 $(EXTFLASH_ERASESIZE) \
 " | bc)

# Calculate application end adress in ext FLASH
 APPLICATION_END = $(shell echo 524288 + $(APPLICATION_SIZE) | bc)


BOARD_SRCS += $(DRIVER_PATH)/src/misc.c 
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_adc.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_dcmi.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_hash.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_hash_md5.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_hash_sha1.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_rng.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_syscfg.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_cryp.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_cryp_aes.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_cryp_des.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_cryp_tdes.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_can.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_crc.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_dac.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_dbgmcu.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_dma.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_exti.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_flash.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_fsmc.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_gpio.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_i2c.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_iwdg.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_pwr.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_rcc.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_rtc.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_sdio.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_spi.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_tim.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_usart.c
BOARD_SRCS += $(DRIVER_PATH)/src/stm32f4xx_wwdg.c

BOARD_SRCS += $(FRAMEWORK_PATH)/Utilities/STM32_EVAL/STM3240_41_G_EVAL/stm324xg_eval.c
BOARD_SRCS += $(FRAMEWORK_PATH)/Utilities/STM32_EVAL/STM3240_41_G_EVAL/stm324xg_eval_lcd.c
#BOARD_SRCS += $(FRAMEWORK_PATH)/Utilities/STM32_EVAL/STM3240_41_G_EVAL/stm324xg_eval_ioe.c

BOARD_SRCS += $(FRAMEWORK_PATH)/Libraries/CMSIS/Device/ST/STM32F4xx/Source/Templates/system_stm32f4xx.c
BOARD_SRCS += $(PLATFORM_PATH)/board/$(BOARD)/startup_stm32f4xx.c

BOARD_INCS += $(DRIVER_PATH)/inc
BOARD_INCS += $(FRAMEWORK_PATH)/Libraries/CMSIS/Include
BOARD_INCS += $(FRAMEWORK_PATH)/Libraries/CMSIS/Device/ST/STM32F4xx/Include
BOARD_INCS += $(FRAMEWORK_PATH)/Project/STM32F4xx_StdPeriph_Templates

BOARD_INCS += $(FRAMEWORK_PATH)/Utilities/STM32_EVAL/STM3240_41_G_EVAL
BOARD_INCS += $(FRAMEWORK_PATH)/Utilities/STM32_EVAL/Common

# Suppress framework warnings
BOARD_CFLAGS += -Wno-parentheses -Wno-unused
