/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_DBG
#define OWL_FILE "stm324xg-eval"

#include <owl/core/owl_debug.h>
#include <owl/core/owl_defs.h>
#include <owl/core/owl_err.h>
#include <owl/core/owl_timer.h>
#include <owl/drivers/owl_board.h>
#include <owl/drivers/owl_gpio.h>
#include <stm32f4xx_rcc.h>
#include <stm32f4xx_pwr.h>
#include <stm32f4xx_exti.h> 
#include <stm32f4xx.h>


const struct gpiomap gpiomap[49] = {
        { "pa0", STM32_CREATE_PIN(GPIOA, 0) },
        { "pa1", STM32_CREATE_PIN(GPIOA, 1) },
        { "pa2", STM32_CREATE_PIN(GPIOA, 2) },
        { "pa3", STM32_CREATE_PIN(GPIOA, 3) },
        { "pa4", STM32_CREATE_PIN(GPIOA, 4) },
        { "pa5", STM32_CREATE_PIN(GPIOA, 5) },
        { "pa6", STM32_CREATE_PIN(GPIOA, 6) },
        { "pa7", STM32_CREATE_PIN(GPIOA, 7) },
        { "pa8", STM32_CREATE_PIN(GPIOA, 8) },
        { "pa9", STM32_CREATE_PIN(GPIOA, 9) },
        { "pa10", STM32_CREATE_PIN(GPIOA, 10) },
        { "pa11", STM32_CREATE_PIN(GPIOA, 11) },
        { "pa12", STM32_CREATE_PIN(GPIOA, 12) },
        { "pa13", STM32_CREATE_PIN(GPIOA, 13) },
        { "pa14", STM32_CREATE_PIN(GPIOA, 14) },
        { "pa15", STM32_CREATE_PIN(GPIOA, 15) },

        { "pb0", STM32_CREATE_PIN(GPIOB, 0) },
        { "pb1", STM32_CREATE_PIN(GPIOB, 1) },
        { "pb2", STM32_CREATE_PIN(GPIOB, 2) },
        { "pb3", STM32_CREATE_PIN(GPIOB, 3) },
        { "pb4", STM32_CREATE_PIN(GPIOB, 4) },
        { "pb5", STM32_CREATE_PIN(GPIOB, 5) },
        { "pb6", STM32_CREATE_PIN(GPIOB, 6) },
        { "pb7", STM32_CREATE_PIN(GPIOB, 7) },
        { "pb8", STM32_CREATE_PIN(GPIOB, 8) },
        { "pb9", STM32_CREATE_PIN(GPIOB, 9) },
        { "pb10", STM32_CREATE_PIN(GPIOB, 10) },
        { "pb11", STM32_CREATE_PIN(GPIOB, 11) },
        { "pb12", STM32_CREATE_PIN(GPIOB, 12) },
        { "pb13", STM32_CREATE_PIN(GPIOB, 13) },
        { "pb14", STM32_CREATE_PIN(GPIOB, 14) },
        { "pb15", STM32_CREATE_PIN(GPIOB, 15) },
      
        { "pc0", STM32_CREATE_PIN(GPIOC, 0) },
        { "pc1", STM32_CREATE_PIN(GPIOC, 1) },
        { "pc2", STM32_CREATE_PIN(GPIOC, 2) },
        { "pc3", STM32_CREATE_PIN(GPIOC, 3) },
        { "pc4", STM32_CREATE_PIN(GPIOC, 4) },
        { "pc5", STM32_CREATE_PIN(GPIOC, 5) },
        { "pc6", STM32_CREATE_PIN(GPIOC, 6) },
        { "pc7", STM32_CREATE_PIN(GPIOC, 7) },
        { "pc8", STM32_CREATE_PIN(GPIOC, 8) },
        { "pc9", STM32_CREATE_PIN(GPIOC, 9) },
        { "pc10", STM32_CREATE_PIN(GPIOC, 10) },
        { "pc11", STM32_CREATE_PIN(GPIOC, 11) },
        { "pc12", STM32_CREATE_PIN(GPIOC, 12) },
        { "pc13", STM32_CREATE_PIN(GPIOC, 13) },
        { "pc14", STM32_CREATE_PIN(GPIOC, 14) },
        { "pc15", STM32_CREATE_PIN(GPIOC, 15) },

        { "ph0", STM32_CREATE_PIN(GPIOH, 0) },
};

int
owlboard_init(void)
{
        GPIO_InitTypeDef GPIO_InitStructure;
        NVIC_InitTypeDef NVIC_InitStructure;
#if !BOOTLOADER
        EXTI_InitTypeDef EXTI_InitStructure;
#endif
        /* GCC startup code does not currently call SystemInit() */
        SystemInit();

#if !BOOTLOADER
        /* Remap the isr_vector for applications that are loaded by the
           bootloader. Otherwise STM32_CONFIG_BOOTLOADER_SIZE will be 0 */
        NVIC_SetVectorTable(NVIC_VectTab_FLASH,
                            STM32_CONFIG_BOOTLOADER_SIZE);
        
#endif
        SysTick_Config(SystemCoreClock / OWL_BOARD_SYSTICK_HZ);

        RCC_HCLKConfig(RCC_SYSCLK_Div1); 
        RCC_PCLK1Config(RCC_HCLK_Div1);           
        RCC_PCLK2Config(RCC_HCLK_Div1); 

        /*!< At this stage the microcontroller clock setting is already
          configured, this is done through SystemInit() function. To 
          reconfigure the default setting of SystemInit() function, refer to
          system_stm32f10x.c file */     

        /* Configure interrupt controller */
        /* Move this to platform init file */
        /* Configure the NVIC Preemption Priority Bits */  
        NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);
       
 	RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOB, ENABLE);
      	RCC_APB1PeriphClockCmd(RCC_APB1Periph_USART3, ENABLE);
        RCC_APB2PeriphClockCmd(RCC_APB2Periph_SYSCFG, ENABLE);

  	/* Connect PB10 to USART3_Tx*/
  	GPIO_PinAFConfig(GPIOB, GPIO_PinSource10, GPIO_AF_USART3);
  	/* Connect PB11 to USART3_Rx*/
  	GPIO_PinAFConfig(GPIOB, GPIO_PinSource11, GPIO_AF_USART3);

  	/* Configure USART Tx as alternate function  */
  	GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
  	GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
  	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;

  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
  	GPIO_Init(GPIOB, &GPIO_InitStructure);

 	 /* Configure USART Rx as alternate function  */
  	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
  	GPIO_Init(GPIOB, &GPIO_InitStructure);
        
        /* Configure USART RX interrupt */
        NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQn;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
        NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
        NVIC_Init(&NVIC_InitStructure);

        /* Setup LEDs */
        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOG, ENABLE);
        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOI, ENABLE);
        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOC, ENABLE);

        owlgpio_cfg(OWL_BOARD_LED1, OWL_GPIO_OUTPUT);
        owlgpio_cfg(OWL_BOARD_LED2, OWL_GPIO_OUTPUT);
        owlgpio_cfg(OWL_BOARD_LED3, OWL_GPIO_OUTPUT);
        owlgpio_cfg(OWL_BOARD_LED4, OWL_GPIO_OUTPUT);
        
        /* Setup User button */
        owlgpio_cfg(OWL_BOARD_BUTTON0, OWL_GPIO_INPUT);
        
        /* Connect EXTI Line9 to PC9 pin */
        SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOG,
                              EXTI_PinSource15);
        
        /* Configure EXTI line9 */
        EXTI_ClearITPendingBit(EXTI_Line15);

        EXTI_InitStructure.EXTI_Line = EXTI_Line15;
        EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
        EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
        EXTI_InitStructure.EXTI_LineCmd = ENABLE;
        EXTI_Init(&EXTI_InitStructure);

        /* Enable and set EXTI15_10 Interrupt to the lowest priority */
        NVIC_InitStructure.NVIC_IRQChannel = EXTI15_10_IRQn;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
        NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
        NVIC_Init(&NVIC_InitStructure);

        
#if !BOOTLOADER
#if defined(OWL_BOARD_HDG205_SPI)
        /* Setup SPI3 */
        RCC_APB1PeriphClockCmd(RCC_APB1Periph_SPI3, ENABLE);
        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOA, ENABLE);
        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOF, ENABLE);

        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SPI_SCL_PIN), 
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SPI_SCL_PIN), 
                         GPIO_AF_SPI3);
        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SPI_MISO_PIN), 
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SPI_MISO_PIN), 
                         GPIO_AF_SPI3);    
        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SPI_MOSI_PIN), 
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SPI_MOSI_PIN), 
                         GPIO_AF_SPI3);

        /* SPI3 SCK */
        GPIO_InitStructure.GPIO_Pin = 
                STM32_PIN(OWL_BOARD_HDG205_SPI_SCL_PIN); 
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;    
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
        GPIO_Init(STM32_PORT(OWL_BOARD_HDG205_SPI_SCL_PIN), 
                  &GPIO_InitStructure); 
                  
        
        /* SPI3 MOSI */
        GPIO_InitStructure.GPIO_Pin = 
                STM32_PIN(OWL_BOARD_HDG205_SPI_MOSI_PIN);
        GPIO_Init(STM32_PORT(OWL_BOARD_HDG205_SPI_MOSI_PIN), 
                  &GPIO_InitStructure); 
        
        /* SPI3 MISO */
        GPIO_InitStructure.GPIO_Pin =  
                STM32_PIN(OWL_BOARD_HDG205_SPI_MISO_PIN);
        GPIO_Init(STM32_PORT(OWL_BOARD_HDG205_SPI_MISO_PIN), 
                  &GPIO_InitStructure);             
  
        /* Configure I/O for chip select */
        owlgpio_cfg(OWL_BOARD_HDG205_SPI_CS_PIN, OWL_GPIO_OUTPUT);
        
        /* Config SPI IRQ pin */
        owlgpio_cfg(OWL_BOARD_HDG205_SPI_IRQ_PIN, OWL_GPIO_INPUT);

        /* Connect EXTI Line9 to PC9 pin */
        SYSCFG_EXTILineConfig(EXTI_PortSourceGPIOC,
                              EXTI_PinSource9);
        
        /* Configure EXTI line9 */
        EXTI_ClearITPendingBit(EXTI_Line9);

        EXTI_InitStructure.EXTI_Line = EXTI_Line9;
        EXTI_InitStructure.EXTI_Mode = EXTI_Mode_Interrupt;
        EXTI_InitStructure.EXTI_Trigger = EXTI_Trigger_Rising_Falling;
        EXTI_InitStructure.EXTI_LineCmd = ENABLE;
        EXTI_Init(&EXTI_InitStructure);

        /* Enable and set EXTI15_10 Interrupt to the lowest priority */
        NVIC_InitStructure.NVIC_IRQChannel = EXTI9_5_IRQn;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
        NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
        NVIC_Init(&NVIC_InitStructure);
#endif /* OWL_BOARD_HDG205_SPI */

#if defined(OWL_BOARD_HDG205_SDIO)
        /* GPIOC and GPIOD Periph clock enable */
        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_GPIOD | 
                               RCC_AHB1Periph_GPIOH, ENABLE);
        
        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SDIO_D0_PIN),
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SDIO_D0_PIN), 
                         GPIO_AF_SDIO);

        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SDIO_D1_PIN),
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SDIO_D1_PIN), 
                         GPIO_AF_SDIO);

        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SDIO_D2_PIN),
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SDIO_D2_PIN), 
                         GPIO_AF_SDIO);

        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SDIO_D3_PIN),
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SDIO_D3_PIN), 
                         GPIO_AF_SDIO);

        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SDIO_SCK_PIN),
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SDIO_SCK_PIN), 
                         GPIO_AF_SDIO);

        GPIO_PinAFConfig(STM32_PORT(OWL_BOARD_HDG205_SDIO_CMD_PIN),
                         STM32_PINSOURCE(OWL_BOARD_HDG205_SDIO_CMD_PIN), 
                         GPIO_AF_SDIO);
        
        /* Configure PC.08, PC.09, PC.10, PC.11 pins: D0, D1, D2, D3 pins */
        GPIO_InitStructure.GPIO_Pin = 
                STM32_PIN(OWL_BOARD_HDG205_SDIO_D0_PIN) |
                STM32_PIN(OWL_BOARD_HDG205_SDIO_D1_PIN) |
                STM32_PIN(OWL_BOARD_HDG205_SDIO_D2_PIN) |
                STM32_PIN(OWL_BOARD_HDG205_SDIO_D3_PIN);
        
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF;
        GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
        GPIO_Init(
                STM32_PORT(OWL_BOARD_HDG205_SDIO_D0_PIN), 
                &GPIO_InitStructure);
        
        /* Configure PD.02 CMD line */
        GPIO_InitStructure.GPIO_Pin = 
                STM32_PIN(OWL_BOARD_HDG205_SDIO_CMD_PIN);
        
        GPIO_Init(
                STM32_PORT(OWL_BOARD_HDG205_SDIO_CMD_PIN), 
                &GPIO_InitStructure);
        
        /* Configure PC.12 pin: CLK pin */
        GPIO_InitStructure.GPIO_Pin = 
                STM32_PIN(OWL_BOARD_HDG205_SDIO_SCK_PIN);
        
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
        
        GPIO_Init(
                STM32_PORT(OWL_BOARD_HDG205_SDIO_SCK_PIN), 
                &GPIO_InitStructure);
        
        /*!< Configure SD_SPI_DETECT_PIN pin: SD Card detect pin */
        GPIO_InitStructure.GPIO_Pin = 
                STM32_PIN(OWL_BOARD_HDG205_SDIO_DETECT_PIN);
          
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
        
        GPIO_Init(
                STM32_PORT(OWL_BOARD_HDG205_SDIO_DETECT_PIN), 
                &GPIO_InitStructure);
        
        /* Enable the SDIO APB2 Clock */
        RCC_APB2PeriphClockCmd(RCC_APB2Periph_SDIO, ENABLE);
        
        /* Enable the DMA2 Clock */
        RCC_AHB1PeriphClockCmd(RCC_AHB1Periph_DMA2, ENABLE);
   
#endif /* OWL_BOARD_HDG205_SDIO */

#endif
        return 0;
}

int 
owlboard_suspend(int wakeup)
{
        /* Not implemented */
        return 0;
}

int 
owlboard_sleep(int wakeup)
{

        /* Not implemented */
        return 0;
}
