MAKE      = make
TGTTYPE   = $(suffix $(TARGET))

RM        = rm -Rf

AR        = $(CROSS_COMPILE)ar
ARFLAGS   += rcs

CPP       = $(CC) -E
CPPFLAGS  += $(WARNINGS) \
            $(PLATFORM_INC_PATH:%=-I%) $(INC_PATH:%=-I%) $(CPP_EXTRA_FLAGS) \
            -iquote..

CC        = $(CROSS_COMPILE)gcc
CFLAGS    += $(OPTIMIZATION) $(C_EXTRA_FLAGS) \
            $(PLATFORM_INC_PATH:%=-Wa,-I%) $(AS_EXTRA_FLAGS) \
            -iquote..

AS        = $(CROSS_COMPILE)as
ASFLAGS   = $(OPTIMIZATION) \
	    $(WARNINGS) \
            $(PLATFORM_INC_PATH:%=-Wa,-I%) $(AS_EXTRA_FLAGS)

LD        = $(CROSS_COMPILE)gcc
LDFLAGS   += $(OPTIMIZATION) \
            $(LINKER_SCRIPT:%=-T%)
LOADLIBES +=
LDLIBS    += $(LIBS:%=-l%)

OBJDUMP   = $(CROSS_COMPILE)objdump

NM        = $(CROSS_COMPILE)nm

OBJCOPY   = $(CROSS_COMPILE)objcopy
LSS       = $(TARGET:$(TGTTYPE)=.lss)
HEX       = $(TARGET:$(TGTTYPE)=.hex)
BIN       = $(TARGET:$(TGTTYPE)=.bin)
SYM       = $(TARGET:$(TGTTYPE)=.sym)
OBJ       = $(TARGET:$(TGTTYPE)=.o)
LIB       = lib$(TARGET:$(TGTTYPE)=.a)
IMG       = $(TARGET:$(TGTTYPE)=.img)
ARX       = $(TARGET:$(TGTTYPE)=.arx)

SIZE      = $(CROSS_COMPILE)size

DOCGEN    = doxygen

RELEASE_CMD = $(TRUNK_PATH)/scripts/mk_release.pl

ARCHIVE   = $(TRUNK_PATH)/tools/archive/archive
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/tools/archive/archive.c
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/tools/archive/Makefile

BOOTFILE   = $(TRUNK_PATH)/tools/bootfile/bootfile
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/tools/bootfile/bootfile.c
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/tools/bootfile/Makefile

FIXEDFS   = $(TRUNK_PATH)/tools/fixedfs/fixedfs
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/tools/fixedfs/fixedfs.c
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/tools/fixedfs/Makefile
# These headers are needed by FIXEDFS, ARCHIVE and BOOTFILE
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/toolkit/owl/util/mtd/archive.h
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/toolkit/owl/util/mtd/bootfile.h
RELEASE_EXTRA_FILES += $(TRUNK_PATH)/toolkit/owl/util/mtd/fixedfs.h

RELEASE_EXTRA_FILES += $(PLATFORM_PATH)/openocd/openocd_reset.cfg
RELEASE_EXTRA_FILES += $(PLATFORM_PATH)/openocd/openocd_debug.cfg
RELEASE_EXTRA_FILES += $(PLATFORM_PATH)/openocd/openocd_upload.cfg

# Extra targets for this arch
openocd:
	$(shell echo "source [find interface/$(OPENOCD_LINK).cfg]" > ocd_target.cfg)
	$(shell echo "source [find target/$(OPENOCD_TARGET).cfg]" >> ocd_target.cfg)
	$(shell echo "set TARGET ../$(OBJ_PATH)/$(TARGET)" >> ocd_target.cfg)

# Reset MCU.
.PHONY: reset
reset: openocd
ifeq (,$(findstring run,$(MAKECMDGOALS)))
 ifeq (,$(findstring program,$(MAKECMDGOALS)))
	$(shell openocd -f $(PLATFORM_PATH)/openocd/openocd_reset.cfg)
 endif
endif

# Debug MCU.
.PHONY: gdb
gdb: openocd
	$(shell openocd -f $(PLATFORM_PATH)/openocd/openocd_debug.cfg)

# Program MCU memory from ELF output file.
.PHONY: program
program: openocd all
	$(shell openocd -f $(PLATFORM_PATH)/openocd/openocd_upload.cfg)

# Find library files we link with. We look for the files in LIBS in
# all the dirs specified in LIB_PATH.
LIB_FILES = $(realpath $(foreach dir,$(LIB_PATH),$(addprefix $(dir)/lib,$(addsuffix .a,$(LIBS)))))
.PHONY: release
release: $(TARGET)
	$(shell echo PATH $(realpath $(TRUNK_PATH)) > release.cfg)
	$(shell echo PATH $(realpath $(EXT_PATH)) >> release.cfg)
	$(shell echo MK_FILES $(realpath $(MK_FILES)) >> release.cfg)
	$(shell echo $(realpath $(RELEASE_EXTRA_FILES)) >> release.cfg)
	$(shell echo $(realpath $(LINKER_SCRIPT)) >> release.cfg)
	$(shell echo $(LIB_FILES) >> release.cfg)
	$(shell cat $(DEPFILES) | sort | uniq | sed 's/.*:\(.*$$\)/\1/' | tr -d '\\' | tr "\n" " " | xargs -n 1 readlink -f | sort | uniq >> release.cfg)
	$(RELEASE_CMD) -n $(basename $(TARGET)) release.cfg

# Create extended listing from target output file.
.PHONY: lss
lss: $(LSS)

# Assembly listing
$(LSS): $(TARGET)
	$(OBJDUMP) -h -S $< > $@

# Create symbol table from target output file.
.PHONY: sym
sym: $(SYM)

# Create symbol table from target output file.
$(SYM): $(TARGET)
	$(NM) -n $< > $@

# Create binary image from ELF output file.
.PHONY: bin
bin: $(BIN)

# Create binary image from ELF output file.
$(BIN): $(TARGET)
	$(OBJCOPY) -O binary $< $@


# NOTE: The 'sizes' target has a lot of unnessecary deps, this is to
#	ensure that this will be performed as the last step of the build.
#	Otherwise the sizes printout might be interleaved by some other build
#	steps.
.PHONY: sizes
sizes: $(OBJFILES) $(TARGET) img arx boot
	@$(SIZE) $(OBJFILES) $(TARGET)
ifeq ($(TGTTYPE),.elf)
	@$(SIZE) -Ax $<
	@$(SIZE) -Bx $<
endif

# Create .o-file of the bootloader binary so we can link it with the
# application. This is to eventually get one binary which contains both
# the bootloader and the application so we can program it through JTAG.
#
# For upgrades the "img" target will be used, this will strip the boot-section.
# BUILD_BOOT is only defiend by bootloader.
.PHONY: boot
boot: bin
ifeq ($(BUILD_BOOT),1)
	test -n "$$(find . -name $(BIN) -a -size -$(BOOTLOADER_SIZE))"
	$(OBJCOPY) -I binary -O elf32-littlearm -B arm --pad-to $(BOOTLOADER_SIZE)  --rename-section .data=.boot $(BIN) $(OBJ)
	$(AR) -rc $(LIB) $(OBJ)
endif

# Build tools needed for .arx-file generation
$(ARCHIVE) : $(ARCHIVE).c
	$(MAKE) -C $(dir $(ARCHIVE))

$(FIXEDFS) : $(FIXEDFS).c
	$(MAKE) -C $(dir $(FIXEDFS))

$(BOOTFILE) : $(BOOTFILE).c
	$(MAKE) -C $(dir $(BOOTFILE))

# Create files for application upgrades by stripping the bootloader from the
# binary.
.PHONY: img
img: $(TARGET)
ifneq ($(BUILD_BOOT),1)
	$(OBJCOPY) -O binary -R .boot $(TARGET) $(IMG)
endif

# Create arx file for upgrade. The application (config.mk) and the board
# (board.mk) will create the ARCHIVE_FLAGS variable so the proper arx file can
# be written. The arx file depends on the layout of internal and external flash.
.PHONY: arx
arx: img $(ARCHIVE) $(FIXEDFS) $(BOOTFILE)
ifneq ($(BUILD_BOOT),1)
 ifdef FIXEDFS_FLAGS
	$(FIXEDFS) $(FIXEDFS_FLAGS)
 endif
	$(BOOTFILE) $(BOOTFILE_FLAGS)
	$(ARCHIVE) $(ARCHIVE_FLAGS)
endif

tags : $(TARGET)
	rm -f ../TAGS
	cat $(DEPFILES) | sort | uniq | sed 's/.*:.*$$//' | tr -d '\\' | tr "\n" " " | xargs -n 1 readlink -f | sort | uniq | xargs etags -a
	cp TAGS ..

all : lss sym bin boot img arx sizes
