#  Copyright (C) 2012, H&D Wireless AB All rights reserved.
#
#  Redistribution and use in source and binary forms, with or without
#  modification, are permitted provided that the following conditions are met:
#
#  1. Redistributions of source code must retain the above copyright notice,
#     this list of conditions and the following disclaimer.
#
#  2. Redistributions in binary form must reproduce the above copyright notice,
#     this list of conditions and the following disclaimer in the documentation
#     and/or other materials provided with the distribution.
#
#  3. The name of H&D Wireless AB may not be used to endorse or promote
#     products derived from this software without specific prior written
#     permission.
#
#  4. This software may only be redistributed and used in connection with an
#     H&D Wireless product.
#
#  THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY EXPRESS OR
#  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
#  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE EXPRESSLY AND
#  SPECIFICALLY DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT,
#  INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
#  (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
#  LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#  ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
#  (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
#  THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

DEFS += -DOWL_TOOLKIT

#$(info FRAMEWORK_PATH:$(FRAMEWORK_PATH))
#$(info BOARD:$(BOARD))
#$(info DEBUG:$(DEBUG))

OS ?= libc

OPTIMIZATION ?= -Os
CFLAGS += -std=gnu99 -Werror
CFLAGS += -Wno-error=unused-but-set-variable -Wno-error=address
CFLAGS += -Wno-error=unused-function
CFLAGS += -Wno-unused-but-set-variable -Wno-address
CFLAGS += -Wno-error=inline

# Configure debugging
#
ifeq (1,$(DEBUG))
 DEFS += -DOWL_DEBUG -DOWL_ASSERT
 CFLAGS += -g
else
 DEFS += -DLWIP_NOASSERT
endif

include $(TRUNK_PATH)/os/$(OS)/os.mk
MK_FILES += $(TRUNK_PATH)/os/$(OS)/os.mk

# To include the bootloader we have to give the --whole-archive directive to
# the linker. This is to avoid the linker from ignoring our bootloader binary
# blob, since it not referenced from anywhere. This is requrired since the
# bootloader blob is an archive and not a plain o-file.
#
# BOOTLOADER_SIZE is defined for each board.
ifeq (1,$(BOOTLOADER))
 DEFS += -DSTM32_CONFIG_BOOTLOADER_SIZE=$(BOOTLOADER_SIZE)
 DEFS += -DSTM32_CONFIG_BOOTLOADER=1
 LIB_PATH += $(PLATFORM_PATH)/bootloader/_$(BOARD)
 # Add bootloader binary to releases
 RELEASE_EXTRA_FILES += $(PLATFORM_PATH)/bootloader/_$(BOARD)/libbootloader-$(BOARD).a
 LDFLAGS += --whole-archive -lbootloader-$(BOARD) --no-whole-archive
endif

# Include path
INC_PATH += $(APP_PATH)
INC_PATH += $(TRUNK_PATH)/wl_api
INC_PATH += $(TRUNK_PATH)/toolkit
INC_PATH += $(PLATFORM_PATH)/owl
INC_PATH += $(PLATFORM_PATH)/util
INC_PATH += $(TRUNK_PATH)/platform
INC_PATH += $(TRUNK_PATH)
INC_PATH += $(FRAMEWORK_PATH)/Utilities/STM32_EVAL
INC_PATH += $(TRUNK_PATH)/wlp_api
INC_PATH += $(TRUNK_PATH)/wlx_api
INC_PATH += $(PLATFORM_PATH)/board/$(BOARD)

# Sources
CSRCS += $(PLATFORM_PATH)/util/core_cm3.c
CSRCS += $(PLATFORM_PATH)/board/$(BOARD)/$(BOARD).c

ifeq ($(BUILD_REV),)
	BUILD_REV := $(shell if [ -x $(TRUNK_PATH)/scripts/revget.sh ]; then cd $(TRUNK_PATH); ./scripts/revget.sh .; else echo -n UNSET; fi)
endif
ifeq ($(BUILD_DATE),)
	BUILD_DATE := $(shell date +"%Y-%m-%d %H:%M:%S")
endif
ifeq ($(BUILD_RELEASE),)
	BUILD_RELEASE := "1.0.0"
endif
DEFS += -DBUILD_REV="\"$(BUILD_REV)\""
DEFS += -DBUILD_DATE="\"$(BUILD_DATE)\""
DEFS += -DBUILD_RELEASE="\"$(BUILD_RELEASE)\""
DEFS += -DBUILD_TARGET=\"$(TARGET:.elf=)\"
