/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */


#define OWL_TRACE OWL_TRACE_ERR

#include <owl/core/owl_debug.h>
#include <owl/core/owl_err.h>
#include <owl/drivers/owl_board.h>

#if BOARD == STM3210E_EVAL
#include <core_cm3.h>
#endif

void
owlboard_mdelay(int ms)
{
        int i;                                  
        volatile int a = 0;
        for (i = 0; i < ms * 5000; i++)         
                a++;                                
}

void 
owl_putc(char c)
{
#if defined(OWL_BOARD_DEBUG_UART)
        /* write a character to the USART */
        USART_SendData(OWL_BOARD_DEBUG_UART, (uint8_t) c);

        /* Loop until the end of transmission */
        while (USART_GetFlagStatus(OWL_BOARD_DEBUG_UART, USART_FLAG_TC) ==
               RESET);

        if (c == '\n') {
                USART_SendData(OWL_BOARD_DEBUG_UART, (uint8_t) '\r');
                while (USART_GetFlagStatus(OWL_BOARD_DEBUG_UART,
                                           USART_FLAG_TC) == RESET);
        }
#endif
}



#if BOARD == STM3210E_EVAL
int
owlboard_init(void)
{
        USART_InitTypeDef USART_InitStructure;
        NVIC_InitTypeDef NVIC_InitStructure;
        
        /* GCC startup code does not currently call SystemInit() */
        SystemInit();
#if !BOOTLOADER
        /* Remap the isr_vector for applications that are loaded by the
           bootloader. Otherwise STM32_CONFIG_BOOTLOADER_SIZE will be 0 */
        NVIC_SetVectorTable(NVIC_VectTab_FLASH,
                            STM32_CONFIG_BOOTLOADER_SIZE);
#endif
        SysTick_Config(SystemCoreClock / 100);

        /*!< At this stage the microcontroller clock setting is already
          configured, this is done through SystemInit() function. To reconfigure
          the default setting of SystemInit() function, refer to
          system_stm32f10x.c file */     

        /* Configure interrupt controller */
        /* Move this to platform init file */
        /* Configure the NVIC Preemption Priority Bits */  
        NVIC_PriorityGroupConfig(NVIC_PriorityGroup_0);

        
        USART_StructInit(&USART_InitStructure);
        STM_EVAL_COMInit(COM1, &USART_InitStructure);
        STM_EVAL_COMInit(COM2, &USART_InitStructure);

        NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQn;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
        NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
        NVIC_Init(&NVIC_InitStructure);

        NVIC_InitStructure.NVIC_IRQChannel = USART2_IRQn;
        NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
        NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
        NVIC_Init(&NVIC_InitStructure);

        return 0;
}
#endif /* BOARD */

#if OWL_CONFIG_DB
const struct db_strmap board_strmap[] = {
        { SPB860, (char *) "spb860" },
        { SPB820, (char *) "spb820" },
        DB_STRMAP_END
};
#endif /* OWL_CONFIG_DB */

const char *
owlboard_str(uint32_t board)
{ 
        /* boards from owl_port.h */
	switch (board) {
        case SPB860: return "spb860";
        case SPB820: return "spb820";
        default: return "unknown";
        };
}

void
owlboard_reset(void)
{
        NVIC_SystemReset();
}
               
