/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_MINIMAL
#define OWL_FILE "owl_gpio"

#include <owl/core/owl_defs.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_err.h>
#include <owl/drivers/owl_gpio.h>

int 
owlgpio_init(void) 
{
        /* No init needed on this platform */
        return 0;
}

int 
owlgpio_cfg(int pin, int flags) 
{
        GPIO_InitTypeDef GPIO_InitStructure;

	owl_info("pin:0x%x flags:0x%x", pin, flags);

        GPIO_InitStructure.GPIO_Pin = STM32_PIN(pin);
        
        /* flag translation from owl to stdlib */
#if STM32_BOARD_FAMILY == STM32_FAMILY_F10X

        if ((flags & OWL_GPIO_OUTPUT) &&
            (flags & (OWL_GPIO_PULL_DOWN | OWL_GPIO_PULL_UP))) {
                owl_err("pull-up and pull-down can only be used on inputs, "
                        "flags:%x", flags);
                return OWL_ERR_PARAM;
        }

        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_2MHz;

        if (flags & OWL_GPIO_INPUT) {
		if (flags & OWL_GPIO_PULL_DOWN) {
			GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPD;
		} else if (flags & OWL_GPIO_PULL_UP) {
                        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
		} else {
                        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
		}

	} else if (flags & OWL_GPIO_OUTPUT) {
                GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	} else {
		return OWL_ERR_PARAM;
	}

#elif STM32_BOARD_FAMILY == STM32_FAMILY_L1XX || STM32_BOARD_FAMILY == STM32_FAMILY_F4XX

 #if STM32_BOARD_FAMILY == STM32_FAMILY_L1XX
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_40MHz;
 #elif STM32_BOARD_FAMILY == STM32_FAMILY_F4XX
        GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
 #endif
        if (flags & OWL_GPIO_INPUT) {
                GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN;
		if (flags & OWL_GPIO_PULL_DOWN) {
			GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
		} else if (flags & OWL_GPIO_PULL_UP) {
                        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
		} else {
                        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
		}
		
	} else if (flags & OWL_GPIO_OUTPUT) {
                GPIO_InitStructure.GPIO_Mode = GPIO_Mode_OUT;
                GPIO_InitStructure.GPIO_OType = GPIO_OType_PP;
                if (flags & OWL_GPIO_PULL_DOWN) {
                        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_DOWN;
                } else if (flags & OWL_GPIO_PULL_UP) {
                        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_UP;
                } else {
                        GPIO_InitStructure.GPIO_PuPd = GPIO_PuPd_NOPULL;
                }

	} else {
		return OWL_ERR_PARAM;
	}
#else
# error
#endif

        GPIO_Init(STM32_PORT(pin), &GPIO_InitStructure);

        return 0;
}

void 
owlgpio_set(int pin)
{
	owl_dbg("pin:0x%x", pin);
        GPIO_SetBits(STM32_PORT(pin), STM32_PIN(pin));
}

void 
owlgpio_clr(int pin)
{ 
	owl_dbg("pin:0x%x", pin);
        GPIO_ResetBits(STM32_PORT(pin), STM32_PIN(pin));
}

int
owlgpio_is_high(int pin)
{
        return GPIO_ReadInputDataBit(STM32_PORT(pin), STM32_PIN(pin));
}

int
owlgpio_parse(const char *str)
{
        unsigned int i;
        for (i = 0; i < owl_array_size(gpiomap); i++) {
                if (owl_os_strcasecmp(str, gpiomap[i].str))
                        continue;
                                
                return gpiomap[i].val;
        }

        return OWL_ERR_PARAM;
}

int
owlgpio_next(int pin)
{
        if (pin < 0)
                return 0;
        
        if ((unsigned) pin < owl_array_size(gpiomap) - 1)
                return pin + 1;

        return OWL_GPIO_PIN_NONE;
}


const char *
owlgpio_str(int pin)
{
        if (pin < 0 || (unsigned) pin >= owl_array_size(gpiomap)) {
                owl_err("pin:%d not found in map", pin);
                return NULL;
        }

        return gpiomap[pin].str;
}

