/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#include <owl/core/owl_defs.h>
#include <owl/core/owl_err.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_shell.h>

#include <stm32/util/stack.h>

extern char _ebss; /* Defined by the linker */
extern char __c_stack_top__; /* Defined by the linker */
extern char *heap_end; /* modified by _sbrk() if libc is used */

/* startup_stm32f10x.c fills stack with 0x66666666 */

static void
stack_f(struct owlsh_ctx *sh, int argc, const char *argv[])
{
        owl_unused(argc);
        owl_unused(argv);

        owlsh_printf(sh, "Begin:   %p\r\n", &__c_stack_top__);
        owlsh_printf(sh, "End:     %p\r\n", &_ebss);
        owlsh_printf(sh, "Size:    %d\r\n", stm32_stack_size());
        owlsh_printf(sh, "Used:    %d\n\r", stm32_stack_usage());
        owlsh_printf(sh, "Max:     %d\n\r", stm32_stack_max());
        owlsh_printf(sh, "\n\r");
}

int
stm32_stack_usage(void)
{
        uint32_t *stack = (uint32_t *) __get_MSP();
        return (uint8_t *) &__c_stack_top__ - (uint8_t *) stack;
}

int
stm32_stack_max(void)
{
        uint32_t *addr;
#if OWL_CONFIG_OS_OWL
        for (addr = (uint32_t *) &_ebss;
#else
        for (addr = (uint32_t *) heap_end;
#endif
             addr < (uint32_t *) &__c_stack_top__;
             addr++) {
                if (*addr != 0x66666666)
                        break;
        }
                 
        return (uint8_t *) &__c_stack_top__ - (uint8_t *) addr;
}

int
stm32_stack_size(void)
{
#if OWL_CONFIG_OS_OWL
        return &__c_stack_top__ - &_ebss;
#else
        return &__c_stack_top__ - heap_end;
#endif
}

const struct owlsh_cmd cmd_stack = {
        "stack", stack_f, "stack info"
};
