/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#include <owl/core/owl_debug.h>
#include <owl/core/owl_err.h>

const char *
owl_error(int err)
{
#if OWL_CONFIG_PERROR
        if (err >= 0)
                return "success";
        
        switch (err) {
        case OWL_FAILURE:
                return "unknown failure";
        case OWL_ERR_PARAM:
                return "invalid parameters";
        case OWL_ERR_MEM:
                return "out of memory/buffers";
        case OWL_ERR_CONN:
                return "not connected";
        case OWL_ERR_STATE:
                return "invalid state";
        case OWL_ERR_PARSE:
                return "parsing error";
        case OWL_ERR_PATH:
                return "path not found";
        case OWL_ERR_TYPE:
                return "incompatible type";
        case OWL_ERR_PERM:
                return "no permission";
        case OWL_ERR_NOTSUPP:
                return "operation not supported";
        case OWL_ERR_RANGE:
                return "out of range"; 
        case OWL_ERR_SIZE:
                return "bad size";
        case OWL_ERR_AGAIN:
                return "retry later";
        case OWL_ERR_BUSY:
                return "busy";
        case OWL_ERR_DISABLED:
                return "disabled";
        case OWL_ERR_PROTO:
                return "protocol error";
        case OWL_ERR_TIMEOUT:
                return "timeout";
        case OWL_ERR_IGNORED:
                return "ignored";
        case OWL_ERR_CANCEL:
                return "canceled by remote";
        case OWL_ERR_SYNC:
                return "sync error";
        case OWL_ERR_ABORT:
                return "aborted";
        case OWL_ERR_INUSE:
                return "already in use";
        default:
                owl_assert(0);
#if !defined(OWL_ASSERT)
                owl_err("unknown error code:%d\n", err);
                return "unknown";
#endif
        };
#else
        owl_unused(err);
        return "";
#endif
}
