/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_OPT_H
#define OWL_OPT_H

/* Default values for build options */


#ifndef BOOTLOADER
# define BOOTLOADER 0
#endif

/* Support for bootloader */
#ifndef OWL_CONFIG_BOOTLOADER
# define OWL_CONFIG_BOOTLOADER 0
#endif

#ifndef OWL_CONFIG_BOOTLOADER_SIZE
# define OWL_CONFIG_BOOTLOADER_SIZE 0
#endif

/* Support to omit parameter checks from functions to save some text size.
   Parameter checks that are omitted are only those that can be considered
   bugs at the caller side, such as providing NULL context pointers etc */
#ifndef OWL_CONFIG_API_CHECK
# define OWL_CONFIG_API_CHECK 1
#endif

/* Set to 1 if building on host platform */
#ifndef OWL_CONFIG_HOST
# define OWL_CONFIG_HOST 0
#endif

/* Support for memory leak debugging */
#ifndef OWL_CONFIG_MEMTRACE
# define OWL_CONFIG_MEMTRACE 0
#endif

/* Support for logging to ram */
#ifndef OWL_CONFIG_RAMLOG
# define OWL_CONFIG_RAMLOG 0
#endif

/* Support for event logging to ram */
#ifndef OWL_CONFIG_EVENTLOG
# define OWL_CONFIG_EVENTLOG 0
#endif

/* Maximum line length for debug traces */
#ifndef OWL_CONFIG_OS_PRINTF_LEN
# define OWL_CONFIG_OS_PRINTF_LEN 160
#endif

/* Use static buffer for owl_os_printf(), this will ensure that full debug
   traces can be printed also when we run out of memory */
#ifndef OWL_CONFIG_OS_PRINTF_STACK
# define OWL_CONFIG_OS_PRINTF_STACK 0
#endif

/* Support for database */
#ifndef OWL_CONFIG_DB
# define OWL_CONFIG_DB 0
#endif

#ifndef OWL_CONFIG_DB_TEXT
# define OWL_CONFIG_DB_TEXT 1
#endif

/* This should be set to a value that is at least equal to the maximal
   length of any database value (in string representation). The max size
   is usualy defined by some blob-data */
#ifndef OWL_CONFIG_DB_CLOUD_BUFSIZE
# define OWL_CONFIG_DB_CLOUD_BUFSIZE 256
#endif

/* File system support */
#ifndef OWL_CONFIG_FS
# define OWL_CONFIG_FS 0
#endif

#ifndef OWL_CONFIG_FS_PIPE
# define OWL_CONFIG_FS_PIPE 0
#endif

#ifndef OWL_CONFIG_FS_LINK
# define OWL_CONFIG_FS_LINK 0
#endif

/* Support for netcat.c */
#ifndef OWL_CONFIG_NETCAT
# define OWL_CONFIG_NETCAT 0
#endif

/* Support for xmodem.c */
#ifndef OWL_CONFIG_XMODEM
# define OWL_CONFIG_XMODEM 1
#endif

/* Support for mtd.h */
#ifndef OWL_CONFIG_MTD
# define OWL_CONFIG_MTD 1
#endif

/* Error strings in owl_err.c, disable this to save some flash */
#ifndef OWL_CONFIG_PERROR
# define OWL_CONFIG_PERROR 1
#endif

/* Number of bytes that can be returned by http documents each seq */
#ifndef OWL_CONFIG_HTTPD_RSPLEN
# define OWL_CONFIG_HTTPD_RSPLEN        1024
#endif

/* Limit size of e.g. querystrings */
#ifndef OWL_CONFIG_HTTPD_PARSER_BUFLEN
# define OWL_CONFIG_HTTPD_PARSER_BUFLEN 512
#endif

/* Shell support, owl_shell.c */
#ifndef OWL_CONFIG_SHELL
# define OWL_CONFIG_SHELL 1
#endif

/* Max line size of owlsh_printf() */
#ifndef OWL_CONFIG_SHELL_PRINTF_LEN
# define OWL_CONFIG_SHELL_PRINTF_LEN 80
#endif

#ifndef OWL_CONFIG_SHELL_PRINTF_STACK
# define OWL_CONFIG_SHELL_PRINTF_STACK 0
#endif

/* Max number of args for shell commands */
#ifndef OWL_CONFIG_SHELL_MAXARGS
# define OWL_CONFIG_SHELL_MAXARGS 12
#endif

/* Max line size of shell commands */
#ifndef OWL_CONFIG_SHELL_CMDLEN
# define OWL_CONFIG_SHELL_CMDLEN  80
#endif

/* Support for compact output of shell commands (owlsh_compact()) */
#ifndef OWL_CONFIG_SHELL_OUTPUT_COMPACT
# define OWL_CONFIG_SHELL_OUTPUT_COMPACT 0
#endif

/* Support for full output of shell commands (!owlsh_compact()). Disabling
   this will save some flash, but all shell commands might not have support for
   compact output */
#ifndef OWL_CONFIG_SHELL_OUTPUT_FULL
# define OWL_CONFIG_SHELL_OUTPUT_FULL 1
#endif

/* Repeat last shell cmd using arrow up */
#ifndef OWL_CONFIG_SHELL_HISTORY
# define OWL_CONFIG_SHELL_HISTORY 0
#endif

/* Support for shell to queue output in case the write function was not able to
   accept all of the data. This is useful if the netshell (netsh.c) is used */
#ifndef OWL_CONFIG_SHELL_OUTPUT_QUEUE
# define OWL_CONFIG_SHELL_OUTPUT_QUEUE 0
#endif

/* Should be defined by a particular board, keep in sync with filesystem */
#ifndef OWL_CONFIG_BOOTFILE_ADDR
# define OWL_CONFIG_BOOTFILE_ADDR 0
#endif

/* Name of the bootfile in the filesystem, used by bootfile.c */
#ifndef OWL_CONFIG_BOOTFILE_NAME
# define OWL_CONFIG_BOOTFILE_NAME "bootfile"
#endif

#ifndef OWL_CONFIG_OS_OWL
# define OWL_CONFIG_OS_OWL 0
#endif

#ifndef OWL_CONFIG_OS_OWL_LWIP
# define OWL_CONFIG_OS_OWL_LWIP 1
#endif

#ifndef OWL_CONFIG_FTP
# define OWL_CONFIG_FTP 0
#endif

#ifndef OWL_CONFIG_CLOUD
# define OWL_CONFIG_CLOUD 0
#endif

#ifndef OWL_CONFIG_CLOUD_AUTH
# define OWL_CONFIG_CLOUD_AUTH 0
#endif

#ifndef OWL_CONFIG_CLOUD_CRYPT
# define OWL_CONFIG_CLOUD_CRYPT 0
#endif

#ifndef OWL_CONFIG_CLOCK
# define OWL_CONFIG_CLOCK 0
#endif

#ifndef OWL_CONFIG_PS
# define OWL_CONFIG_PS 1
#endif

#ifndef OWL_CONFIG_NTP
# define OWL_CONFIG_NTP 0
#endif

#ifndef OWL_CONFIG_WATCHDOG
# define OWL_CONFIG_WATCHDOG 0
#endif

#ifndef OWL_CONFIG_LOG
# define OWL_CONFIG_LOG 0
#endif

#ifndef OWL_CONFIG_LOG_NAME
# define OWL_CONFIG_LOG_NAME "log"
#endif

#ifndef OWL_CONFIG_TIMER_STATS
# define OWL_CONFIG_TIMER_STATS 0
#endif

#ifndef OWL_CONFIG_TIMER_SEC
# define OWL_CONFIG_TIMER_SEC 0
#endif

#ifndef OWL_CONFIG_THREAD
# define OWL_CONFIG_THREAD 0
#endif

/* Allow usage of owlthread_craete() in toolkit code */
#ifndef OWL_CONFIG_THREAD_DYNAMIC
# define OWL_CONFIG_THREAD_DYNAMIC 0
#endif

#ifndef OWL_CONFIG_SOCKET_BSD
# define OWL_CONFIG_SOCKET_BSD 0
#endif

/* Additional \r will be added to \n for all traces. This is useful if any
   third party debug traces are routed through owl_os_printf() */
#ifndef OWL_CONFIG_OS_CRLF
# define OWL_CONFIG_OS_CRLF 0
#endif

/* Bootloader printing dots, Should be defined in board.mk. */
#ifndef OWL_CONFIG_BOOTLOADER_DOTS
# define OWL_CONFIG_BOOTLOADER_DOTS 1
#endif

/* Bootloader wait time */
#ifndef OWL_CONFIG_BOOTLOADER_MS
# define OWL_CONFIG_BOOTLOADER_MS 1000
#endif

/* Use battery supervision */
#ifndef OWL_CONFIG_VBAT
# define OWL_CONFIG_VBAT 0
#endif

/* Enable spi slave */
#ifndef OWL_CONFIG_SPI_SLAVE
# define OWL_CONFIG_SPI_SLAVE 0
#endif

#ifndef OWL_CONFIG_HIBERNATE
# define OWL_CONFIG_HIBERNATE 0
#endif

/* Reverse discover */
#ifndef OWL_CONFIG_BROWSER
# define OWL_CONFIG_BROWSER 0
#endif

#ifndef OWL_CONFIG_I2C_MASTER
# define OWL_CONFIG_I2C_MASTER 1
#endif

#ifndef OWL_CONFIG_I2C_SLAVE
# define OWL_CONFIG_I2C_SLAVE 0
#endif

/* Used by no_free */
#ifndef OWL_CONFIG_HEAP_SIZE
# define OWL_CONFIG_HEAP_SIZE 8192
#endif

/* Used by led.h */
#ifndef OWL_CONFIG_LED_MAX
# define OWL_CONFIG_LED_MAX 2
#endif


#endif /* OWL_OPT_H */
