/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */


/*
 *  OS abstraction layer.
 *  Copyright (C) 2010 HD Wireless AB
 *
 */

#ifndef OWL_OS_H
#define OWL_OS_H

#include <owl_cc.h>

#if defined(OWL_CONFIG_MEMTRACE) && OWL_CONFIG_MEMTRACE
# include <owl/core/owl_memtrace.h>
# define owl_os_alloc(size)                             \
        owl_memtrace_alloc(size, __FILE__, __LINE__)
# define owl_os_alloc_ext(size, flags)                          \
        owl_memtrace_alloc_ext(size, flags, __FILE__, __LINE__)
# define owl_os_zalloc(size)                            \
        owl_memtrace_zalloc(size, __FILE__, __LINE__)
# define owl_os_realloc(ptr, size)                              \
        owl_memtrace_realloc(ptr, size, __FILE__, __LINE__)
# define owl_os_free(p)                         \
        owl_memtrace_free(p)
# define owl_os_strdup(p)                               \
        owl_memtrace_strdup(p, __FILE__, __LINE__)
# define owl_os_strndup(p, n)                           \
        owl_memtrace_strndup(p, n, __FILE__, __LINE__)

#elif defined(OWL_CONFIG_MEMCHECK) && OWL_CONFIG_MEMCHECK
# include <owl/core/owl_memcheck.h>
# define owl_os_alloc(size)                             \
        owl_memcheck_alloc(size, __FILE__, __LINE__)
# define owl_os_alloc_ext(size, flags)                          \
        owl_memcheck_alloc_ext(size, flags, __FILE__, __LINE__)
# define owl_os_zalloc(size)                            \
        owl_memcheck_zalloc(size, __FILE__, __LINE__)
# define owl_os_realloc(ptr, size)                              \
        owl_memcheck_realloc(ptr, size, __FILE__, __LINE__)
# define owl_os_free(p)                         \
        owl_memcheck_free(p)
# define owl_os_strdup(p)                               \
        owl_memcheck_strdup(p, __FILE__, __LINE__)
# define owl_os_strndup(p, n)                           \
        owl_memcheck_strndup(p, n, __FILE__, __LINE__)
#else
# define owl_os_alloc owl_mem_alloc
# define owl_os_alloc_ext owl_mem_alloc_ext
# define owl_os_zalloc owl_mem_zalloc
# define owl_os_realloc owl_mem_realloc
# define owl_os_free owl_mem_free
# define owl_os_strdup owl_mem_strdup
# define owl_os_strndup owl_mem_strndup
#endif

int owl_os_init(void);
void owl_os_set_error_cb(void (*cb)(int err));

void *owl_mem_alloc(size_t size);
void *owl_mem_alloc_ext(size_t size, int flags);
void *owl_mem_zalloc(size_t size);
void *owl_mem_realloc(void *ptr, size_t size);
void owl_mem_free(void *p);
char *owl_mem_strdup(const char *s);
char *owl_mem_strndup(const char *s, size_t n);

void *owl_os_memcpy(void *dst, const void *src, size_t n);
void *owl_os_memset(void *s, int c, size_t n);
void *owl_os_memmove(void *dst, const void *src, size_t n);
size_t owl_os_strlen(const char *s);
char *owl_os_strncpy(char *dst, const char *src, size_t n);
int owl_os_strncmp(const char *s1, const char *s2, size_t n);
int owl_os_strncasecmp(const char *s1, const char *s2, size_t n);
int owl_os_strcmp(const char *s1, const char *s2);
char *owl_os_strcpy(char *dst, const char *src);
char *owl_os_strcat(char *dst, const char *src);
int owl_os_memcmp(const void *s1, const void *s2, size_t n);
long int owl_os_strtol(const char *nptr, char **endptr, int base);
#define owl_os_atoi(p) owl_os_strtol(p, NULL, 10)
char *owl_os_strchr(const char *s, int c);
char *owl_os_strrchr(const char *s, int c);
int owl_os_strcasecmp(const char *s1, const char *s2);

char *owl_os_strstr(const char *haystack, const char *needle);
char *owl_os_strcasestr(const char *haystack, const char *needle);
char *owl_os_strtok(char *s, const char *delim);

int owl_os_snprintf(char *str, size_t size, const char *format, ...);
int owl_os_vsnprintf(char *str, size_t size, const char *format, va_list ap);
    
int owl_os_vprintf(const char *format, va_list arg); /* debug only */
int owl_os_printf(const char *format, ...); /* debug only */

extern const struct owlsh_cmd cmd_heap;

#endif /* OWL_OS_H */
