/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_SHELL_H
#define OWL_SHELL_H

#include <owl/core/owl_defs.h>

struct owlsh_ctx;

#define OWLSH_NOPROMPT    (1 << 0)
#define OWLSH_NOECHO      (1 << 1)
#define OWLSH_ESCAPECHARS (1 << 2)
#define OWLSH_COMPACT     (1 << 3)

struct owlsh_cmd {
        const char *label;
        void (*cmd_f)(struct owlsh_ctx *shell, int args, const char *argv[]);
        const char *help;
};

struct owlsh_ctx *owlsh_create(
        const char *prompt,
        int (*write_f)(void *ctx, const void *data, int len), void *ctx,
        int flags);
int owlsh_addcmd(struct owlsh_ctx *priv, const struct owlsh_cmd *cmd,
                 void *ctx);
int owlsh_addcmd_ext(struct owlsh_ctx *priv, const struct owlsh_cmd *cmd,
                     void *ctx, const char *instance);
void owlsh_defcmd(struct owlsh_ctx *priv, const struct owlsh_cmd *cmd, void *ctx);
void owlsh_destroy(struct owlsh_ctx *priv);
void owlsh_key(struct owlsh_ctx *priv, char key);
void *owlsh_data(struct owlsh_ctx *priv);
const char *owlsh_instance(struct owlsh_ctx *priv);

int owlsh_compact(struct owlsh_ctx *priv);
int owlsh_perr(struct owlsh_ctx *priv, int err);
int owlsh_printf(struct owlsh_ctx *priv, const char *fmt, ...)

#if __GNUC__
    __attribute__((format(printf, 2, 3)))
#endif
    ;
int owlsh_input(struct owlsh_ctx *priv, const char *buf, int len);
char *owlsh_joinargs(int argc, const char *argv[]);
void owlsh_hexdump(struct owlsh_ctx *priv, const char *title, const void *pos,
                   int len);


#if OWL_CONFIG_SHELL_OUTPUT_FULL && OWL_CONFIG_SHELL_OUTPUT_COMPACT
# define owlsh_usage(sh, ...) do {                  \
                if (owlsh_compact(sh))                  \
                        owlsh_perr(sh, OWL_ERR_USAGE);  \
                else                                    \
                        owlsh_printf(sh, __VA_ARGS__);         \
        } while (0)

#elif OWL_CONFIG_SHELL_OUTPUT_FULL
# define owlsh_usage(sh, ...) owlsh_printf(sh, __VA_ARGS__)
#elif OWL_CONFIG_SHELL_OUTPUT_COMPACT
# define owlsh_usage(sh, ...) owlsh_perr(sh, OWL_ERR_USAGE); 
#endif /* OWL_CONFIG_SHELL_OUTPUT_FULL && OWL_CONFIG_SHELL_OUTPUT_COMPACT */

extern const struct owlsh_cmd cmd_help;

#endif
