/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_BOARD_H
#define OWL_BOARD_H

#include <owl/core/owl_shell.h>
#if OWL_CONFIG_DB
# include <owl/util/db/db.h>
#endif /* OWL_CONFIG_DB */
#if OWL_CONFIG_CLOUD
# include <owl/util/cloud.h>
#endif /* OWL_CONFIG_CLOUD */

/**
 * @brief Board initialization and board-specific data.
 *
 *
 */

struct owlboard_data; /* Details of this struct should be defined by the
                         particular implementation of owl_board.h */


/**
 * Initialize board pinout and components. This function should be called before
 * any other interface from owl/drivers is used.
 *
 * Upon successful return, the board pins will be initialization according to
 * the board description supplied. The structure of the board description
 * used during owlboard_init() is platform-specific, for some platform each
 * board has a different include file which defines pins etc.
 *
 * @return 0 on success, error code from owl_err.h on error.
 *
 */
int owlboard_init(void);


/**
 * Delay execution by busy-waiting for the specified number of ms.
 *
 *
 */
void owlboard_mdelay(int ms);


/**
 * Get timestamp in microsecond resolution.
 *
 */
uint32_t owlboard_get_us(void);

/**
 * Suspend the board for a specific amount of time. The board might wakeup
 * earlier if an external event/interrupt occured. The the of event that will
 * wakeup the board is board-specific.
 *
 * @param wakeup is the the max number of ms to sleep. If 0, the board will
 *      sleep until an event occurs.
 *
 * @return the number of ms actually slept.
 *
 */
int owlboard_suspend(int wakeup);


/**
 * Enter sleep mode for a specific amount of time. The board might wakeup
 * earlier if an internal or external event/interrupt occured. All interrupts
 * will wakeup the board.
 *
 * @param wakeup is the the max number of ms to sleep. If 0, the board will
 *      sleep until an event occurs.
 *
 * @return the number of ms actually slept.
 *
 */
int owlboard_sleep(int wakeup);



/**
 * Get the board specific data, such as flash memory devices. The
 * owlboard_data struct is initialized during owlboard_init().
 *
 * @return board-specific data.
 */
struct owlboard_data *owlboard_data(void);


/**
 * Convert board id to string.
 *
 */
const char *owlboard_str(uint32_t board);

/**
 * Software reset of board.
 *
 */
void owlboard_reset(void);



void owlboard_putc(char c);

void owlboard_puts(const char *str);

void owlboard_set_reset_cb(void (*cb)(void *ctx), void *ctx);

# if OWL_CONFIG_CLOUD
extern const struct cloud_cmd cloud_reset;
# endif /* OWL_CONFIG_CLOUD */

# if OWL_CONFIG_DB
extern const struct db_strmap board_strmap[];
# endif /* OWL_CONFIG_DB */

/**
 * Perform operations such as reset, suspend etc on the board.
 *
 */
extern const struct owlsh_cmd cmd_board;

#endif /* OWL_BOARD_H */
