/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_SPI_H
#define OWL_SPI_H

#include <owl/core/owl_shell.h>

#define OWL_SPI_MODE_MASTER     0
#define OWL_SPI_MODE_SLAVE      1

#define OWL_SPI_TRANSFER_MODE_0 0
#define OWL_SPI_TRANSFER_MODE_1 1
#define OWL_SPI_TRANSFER_MODE_2 2
#define OWL_SPI_TRANSFER_MODE_3 3

/**
 * Initialize SPI bus
 *
 *
 */
int owlspi_init(void *bus, int mode);


/**
 *
 *
 */ 
int owlspi_setup_cs(void *bus, int cs, int freq, int xmode, int width);


/**
 * Select device on bus
 *
 */
void owlspi_claim(void *bus, int cs);


/**
 * Release device from bus
 *
 */
void owlspi_release(void *bus, int cs);


/**
 * Send and recv data from currently selected device
 *
 */
int owlspi_exchange(void *bus, const void *out, void *in, int len);
int owlspi_write(void *bus, const void *buf, int len);
int owlspi_read(void *bus, void *buf, int len);


/**
 * Exchange data with additional flags.
 *
 * This function can call owlspi_exchange() directly by most implementations.
 * However, some SPI HW might require that we specify that the current transfer
 * is the last for this chip select. This is most often found in SPI HW that
 * controls the SPI CS itself. Then this function can be used to ensure that
 * the CS pin does not get deasserted between two transfers.
 */ 
#define OWL_SPI_FLAG_LAST_XFER (1 << 0)
int owlspi_exchange_ext(void *bus, const void *out, void *in, int len,
                        int flags);

int owlspi_poll(void *bus);

extern const struct owlsh_cmd cmd_spi;


#endif /* OWL_SPI_H */
         
