/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef OWL_UART_H
#define OWL_UART_H

#include <owl/core/owl_defs.h>

enum uart_parity {
        UART_PARITY_NONE = 0,
        UART_PARITY_EVEN,
        UART_PARITY_ODD,
        UART_PARITY_MARK,
        UART_PARITY_SPACE
};

enum uart_stopbits {
        UART_STOPBITS_1 = 1,
        UART_STOPBITS_2
};

enum uart_databits {
        UART_DATABITS_5 = 5,
        UART_DATABITS_6,
        UART_DATABITS_7,
        UART_DATABITS_8,
        UART_DATABITS_9
};

#define UART_BAUDRATE_300       300
#define UART_BAUDRATE_1200      1200
#define UART_BAUDRATE_2400      2400
#define UART_BAUDRATE_4800      4800
#define UART_BAUDRATE_9600      9600
#define UART_BAUDRATE_19200     19200
#define UART_BAUDRATE_38400     38400
#define UART_BAUDRATE_57600     57600
#define UART_BAUDRATE_115200    115200
#define UART_BAUDRATE_230400    230400
#define UART_BAUDRATE_460800    460800
#define UART_BAUDRATE_500000    500000
#define UART_BAUDRATE_576000    576000
#define UART_BAUDRATE_921600    921600
#define UART_BAUDRATE_1000000   1000000
#define UART_BAUDRATE_1500000   1500000
#define UART_BAUDRATE_2000000   2000000
#define UART_BAUDRATE_2500000   2500000
#define UART_BAUDRATE_3000000   3000000
#define UART_BAUDRATE_3500000   3500000
#define UART_BAUDRATE_4000000   4000000

int owluart_init(void *uart,
              uint32_t baudrate, enum uart_databits databits,
              enum uart_parity parity, enum uart_stopbits stopbits,
              int rtscts);
void owluart_outb(void *uart, int c);
int owluart_inb(void *uart, unsigned int timeout);
int owluart_poll(void *uart);
int owluart_exchange(void *uart, const void *tx, void *rx, int len);
int owluart_read(void *uart, void *buf, int len);
int owluart_write(void *uart, const void *buf, int len);
const char *owluart_parity(enum uart_parity parity);
int owluart_set_bufsize(void *uart, int bufsize);

int owluart_ioctl(void *uart, int req, void *param);
#define OWLUART_IOC_R 0x00
#define OWLUART_IOC_W 0x80

/* param: int *; set size of input buffer */
#define OWLUART_IOC_RINGBUF (OWLUART_IOC_W | 0x0) 

/* param: int *; set mode (rs232, rs485, etc) */
#define OWLUART_IOC_MODE (OWLUART_IOC_W | 0x1)
#define OWLUART_MODE_RS232 0
#define OWLUART_MODE_RS485 1
#define OWLUART_MODE_SPI_SLAVE 2


#endif
