/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 *
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_ERR
#define OWL_FILE "mem"

#include <owl/core/owl_defs.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_shell.h>
#include <owl/core/owl_os.h>
#include <owl/core/owl_err.h>
#include <owl/os/owl_mem.h>
#if OWL_CONFIG_MEMTRACE
# include <owl/core/owl_memtrace.h>
#endif

#if OWL_CONFIG_THREAD
# include <owl/core/owl_thread.h>
static owlmutex_t lock;
#endif /* OWL_CONFIG_THREAD */

#include <stdlib.h>

int
owl_mem_init(void)
{
#if OWL_CONFIG_THREAD
	if (owlmutex_create(&lock) < 0)
                return OWL_FAILURE;
#endif /* OWL_CONFIG_THREAD */

#if OWL_CONFIG_MEMTRACE
        owl_memtrace_init();
#endif
        return 0;
}

void *
owl_mem_alloc(size_t size)
{
        void *p;
#if OWL_CONFIG_THREAD
	owlmutex_lock(&lock);
#endif /* OWL_CONFIG_THREAD */
        if ((p = malloc(size)) == NULL) {
		owl_err("could not alloc size:%d bytes", size);
        }
#if OWL_CONFIG_THREAD
	owlmutex_unlock(&lock);
#endif /* OWL_CONFIG_THREAD */
        owl_dbg("p:%p size:%d", p, size);
        return p;
}

void *
owl_mem_zalloc(size_t size)
{
        void *p;
        if ((p = owl_mem_alloc(size)))
                owl_os_memset(p, 0, size);

        return p;
}

void *
owl_mem_realloc(void *ptr, size_t size)
{
        void *p;
#if OWL_CONFIG_THREAD
	owlmutex_lock(&lock);
#endif /* OWL_CONFIG_THREAD */
        if ((p = realloc(ptr, size)) == NULL) {
                owl_err("could not realloc ptr:%p size:%d", ptr, size);
        }
#if OWL_CONFIG_THREAD
	owlmutex_unlock(&lock);
#endif /* OWL_CONFIG_THREAD */
        owl_dbg("realloc ptr:%p p:%p size:%d", ptr, p, size);
        return p;
}

void
owl_mem_free(void *p)
{
        owl_dbg("p:%p");
#if OWL_CONFIG_THREAD
	owlmutex_lock(&lock);
#endif /* OWL_CONFIG_THREAD */
        free(p);
#if OWL_CONFIG_THREAD
	owlmutex_unlock(&lock);
#endif /* OWL_CONFIG_THREAD */
}

