/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_ERR

#include <owl/core/owl_defs.h>
#include <owl/core/owl_os.h>
#include <owl/core/owl_err.h>
#include <owl/drivers/owl_board.h>

static void
board_f(struct owlsh_ctx *sh, int argc, const char *argv[])
{
#if OWL_CONFIG_SHELL_OUTPUT_FULL
        const char *usage = "usage: board <suspend|sleep|reset>\r\n";
        const char *usage_suspend = "usage: board suspend <wakeup>\r\n";
        const char *usage_sleep = "usage: board sleep <wakeup>\r\n";
#endif /* OWL_CONFIG_SHELL_OUTPUT_FULL */
        int wakeup;
        
        if (argc < 2) {
                owlsh_usage(sh, "%s", usage);
                return;
        }
        
        if (!owl_os_strcmp(argv[1], "suspend")) {
                if (argc < 3) {
                        owlsh_usage(sh, "%s", usage_suspend);
                        return;
                }
                wakeup = owl_os_atoi(argv[2]);
                wakeup = owlboard_suspend(wakeup);
                owlsh_printf(sh, "suspended for %d ms\r\n", wakeup);
        } else if (!owl_os_strcmp(argv[1], "sleep")) {
                if (argc < 3) {
                        owlsh_usage(sh, "%s", usage_sleep);
                        return;
                }
                wakeup = owl_os_atoi(argv[2]);
                wakeup = owlboard_sleep(wakeup);
                owlsh_printf(sh, "slept for %d ms\r\n", wakeup);
        } else if (!owl_os_strcmp(argv[1], "reset")) {
                owlboard_reset();
                /* Unreachable */
        } else {
                owlsh_usage(sh, "%s", usage);
        }
}

const struct owlsh_cmd cmd_board = {
        "board", board_f, "board control"
};


#if OWL_CONFIG_CLOUD
static int
reset_recv_cb(struct cloud_req *cloud_req)
{
        owl_unused(cloud_req);
        owlboard_reset();
        return 0;
}

const struct cloud_cmd cloud_reset = { CLOUD_ID_RESET, reset_recv_cb };
#endif /* OWL_CONFIG_CLOUD */
