/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 *
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef ARCHIVE_H
#define ARCHIVE_H

#if !OWL_CONFIG_HOST
# include <owl/core/owl_defs.h>
# include <owl/core/owl_shell.h>
# include <owl/util/mtd/mtd.h>
#endif
/* NOTE: This file is included by PC applications that creates archives */


struct archive_hdr {
#define ARCHIVE_MAGIC 0x1234dbca
        uint32_t magic; /* ARCHIVE_MAGIC */
        uint32_t flags; /* unused */
        uint32_t adler32;
        uint32_t tag;

        uint32_t addr;
        uint32_t len;
        uint32_t tot_len; /* length of chain of hdrs */
        uint8_t dev;
        uint8_t pad[3]; /* unused */
}; /* sizeof: 32 */


#if !OWL_CONFIG_HOST


struct archive_ctx;

/**
 * Create archive.
 *
 *
 */
struct archive_ctx *archive_init(struct mtd_dev *mtd,
                                 int (*tag_f)(void *ctx, uint32_t tag),
                                 void *ctx);

/**
 * Finalize archive and free resources.
 *
 */
int archive_finish(struct archive_ctx *priv);

/**
 * Finalize archive and free resources.
 *
 */

/**
 * Free resources.
 *
 */
void archive_abort(struct archive_ctx *priv);

/**
 * Deliver archive data and store it according to the received archive_hdr.
 * Typically called from e.g. xmodem or netcat recv callbacks.
 *
 * The mtd interface will be used to store the data according to the device and
 * addr information given in the archive_hdr.
 *
 *
 */
int
archive_recv(struct archive_ctx *priv, const char *data, int len);


/**
 * Perform firmware upgrade using XMODEM, NETCAT or FTP. In case netcat is used
 * (nrecv) the TCP port number is 2000.
 *
 */
extern const struct owlsh_cmd cmd_archive;

extern const struct cloud_cmd cloud_archive;

#endif /* !OWL_CONFIG_HOST */

#endif /* ARCHIVE_H */

