/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef FIXEDFS_H
#define FIXEDFS_H

#if !OWL_CONFIG_HOST
# include <owl/util/mtd/ftl.h>

struct fixedfs_desc {
        const char *name;
        uint32_t size;
#define FIXEDFS_EXTERNAL (1 << 0) /* block is external */
        uint8_t flags;
};

/**
 * Cretate a filesystem on top of an FTL.
 *
 * @param ftl is the flash translation layer to use
 * @param desc is an array of block descriptions to use. These will define the
 *        size and other properties of each block. If the FAT stored in flash
 *        does not match the block properties, then the file system will be
 *        considered invalid.
 * @param count is the array length of desc[]
 *
 */
struct fs *fixedfs_create(struct ftl *ftl, const struct fixedfs_desc *desc,
                          int count);

void fixedfs_destroy(struct fs *fs);

extern const struct owlsh_cmd cmd_fixedfs;

#endif /* !OWL_CONFIG_HOST */


/* Filesystem info that is global to all files */
struct fixedfs_hdr {
#define FIXEDFS_MAGIC 0xdeadbeef
        uint32_t magic;  /* 0xdeadbeef */
        uint32_t adler32; /* Checksum of fat */
        uint32_t blkcnt;
};

/* Filesystem info for a specific file */
struct fixedfs_block {
#define FIXEDFS_FILE_LEN 15 /* Keeping the name 15 chars allows us to fit name +
                               flags into 16 bytes, to avoid unncessary
                               padding */
        char name[FIXEDFS_FILE_LEN];

#define block_is_free(b) (owl_bit((b)->flags, FIXEDFS_FREE))
#define FIXEDFS_FREE        (1 << 0)
#define FIXEDFS_UNMANAGED   (1 << 1) /* no checksum/fat */
        uint8_t flags;

        uint32_t size;
        uint32_t adler32;

#define BLOCK_NONE ((uint16_t) -1)
#define block_is_frag(b) ((b)->next != BLOCK_NONE || (b)->prev != BLOCK_NONE)
#define block_is_head(b) ((b)->prev == BLOCK_NONE)
#define block_is_tail(b) ((b)->next == BLOCK_NONE)
        uint16_t next;
        uint16_t prev;
};

struct fixedfs_fat {
        struct fixedfs_hdr hdr;
        struct fixedfs_block blocks[]; /* priv->blkcnt */
};


#endif /* FIXEDFS_H */
