/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 *
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_DBG
#define OWL_FILE "wlp_loopback"

#include <owl/core/owl_defs.h>
#include <owl/core/owl_shell.h>
#include <owl/core/owl_err.h>
#include <owl/core/owl_debug.h>
#include <owl/core/owl_timer.h>

#include <pico/wlp_shell.h>

#include <wlp_loopback.h>

struct loopback_ctx {
        struct owltmr_tmo tmo;
        struct owlsh_ctx *sh;
        int count;
        int len;
        uint8_t data[];
};

static void loopback_f(struct owlsh_ctx *sh, int argc, const char *argv[]);

static void timer_cb(struct owltmr_tmo *tmo, void *ctx);
static void loop_cb(void *ctx, void *data, int len);

const struct owlsh_cmd cmd_loopback = {
        "loopback", loopback_f, "wlp_loopback()"
};

static void
loopback_f(struct owlsh_ctx *sh, int argc, const char *argv[])
{
	int i, len;
        int count = 1;
        struct loopback_ctx *priv;
	const char *usage = "usage: loopback <len> [count]\r\n";
	
	if (argc < 2) {
		owlsh_printf(sh, "%s", usage);
		return;
	}

	len = owl_os_strtol(argv[1], NULL, 0);
        if (argc > 2)
                count = owl_os_strtol(argv[2], NULL, 0);
        
        if ((priv = owl_os_zalloc(sizeof(*priv) + len)) == NULL) {
		owl_err("could not alloc ctx");
		owlsh_perr(sh, OWL_ERR_MEM);
		return;
	}
        priv->sh = sh;
	priv->len = len;
        priv->count = count;
        
	for (i = 0; i < len; i++)
                priv->data[i] = i % 256;

        wlp_set_loopback_cb(loop_cb, sh);
        owltmr_sched(&priv->tmo, 0, 0, timer_cb, priv);
}

static void
loop_cb(void *ctx, void *data, int len)
{
        int i;
        int fail = 0;
        uint8_t *p = data;
        struct owlsh_ctx *sh = ctx;

        for (i = 0; i < len; i++) {
                if (p[i] != i % 256)
                        fail++;
        }

        if (fail)
                owlsh_printf(sh, "x");
        else
                owlsh_printf(sh, ".");
}

static void
timer_cb(struct owltmr_tmo *tmo, void *ctx)
{
        struct loopback_ctx *priv = ctx;
        int err;
        
        if (!(err = wlp_loopback(priv->data, priv->len))) {
                owlsh_printf(priv->sh, ".");
                priv->count--;
        }

        if (!priv->count) {
                owl_os_free(priv);
        } else {
                owltmr_modtmo(tmo, 0);
        }
}
