/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 *
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 *
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#define OWL_TRACE OWL_TRACE_DATA
#define OWL_FILE "pico/shell/mgmt"

#include <owl/core/owl_defs.h>
#include <owl/core/owl_shell.h>
#include <owl/core/owl_err.h>
#include <owl/core/owl_debug.h>

#include <wlp_api.h>

#include <pico/wlp_shell.h>

/*
 * Evaluate the owl pico API (wlp_api) using the wlp_shell.
 *
 * The functions provided by the wlp_api can be invoked from a command line,
 * using a one-to-one mapping between commands and api function wherever
 * possible.
 *
 *
 *
 *
 */

static void heartbeat_f(struct owlsh_ctx *sh, int argc, const char *argv[]);

const struct owlsh_cmd cmd_heartbeat = {
        "heartbeat", heartbeat_f, "wlp_heartbeat()"
};

static void
heartbeat_cb(void *ctx)
{
        struct owlsh_ctx *sh = ctx;

        owl_info("enter");
        owlsh_printf(sh, "got heartbeat\r\n");
}

static void
heartbeat_f(struct owlsh_ctx *sh, int argc, const char *argv[])
{
        owl_unused(argc);
        owl_unused(argv);
        
        int res;

        res = wlp_heartbeat(heartbeat_cb, sh);
        owlsh_perr(sh, res);
        return;
}
