/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_HPI_H
#define WLP_HPI_H

/* Host Processor Interface.
 *
 * These functions will be called by wlp_api.c when communication with the
 * Wi-Fi device should be performed. HPI can be implemented to support UART,
 * SPI and SDIO interfaces.
 *
 */

/**
 * Initialize the HPI interface.
 *
 */
int hpi_init(int (*exchange_cb)(void *ctx, const void *tx, void *rx, int len),
             void (*select_cb)(void *ctx, int select),
             void (*cmd_cb)(void *ctx, uint8_t idx, uint32_t arg,
                            uint32_t *rsp, const void *data, int len),
             int (*poll_cb)(void *ctx),
             void *ctx, int flags);

/**
 * Shutdown the HPI interface.
 *
 */
void hpi_shutdown(void);

/**
 * Add HPI header
 *
 */
int hpi_process_tx(void *data, int len);

/**
 * Send fragment of data on the HPI interface.
 *
 */
int hpi_tx(void *data, int len);

/**
 * Receive data on the HPI interface.
 *
 */
int hpi_rx(void *data, int len);

/**
 * Poll rx data when not using IRQ.
 *
 */
void hpi_poll(void);

#endif /* WLP_HPI_H */
