/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_PROTO_H
#define WLP_PROTO_H

#include <wlp_api.h>
#include <wlp_rftest.h>

#include <hdg205/wlp_hdg205.h>

/* Message types */
#define WLP_TYPE_REQ  (0 << 12)
#define WLP_TYPE_CFM  (1 << 12)
#define WLP_TYPE_IND  (2 << 12)
#define WLP_TYPE_MASK (0xf << 12)
#define WLP_TYPE(id) (id & WLP_TYPE_MASK)

/* Message id's */
#define WLP_ID_INIT           0
#define WLP_ID_RESET          1
#define WLP_ID_LOOPBACK       2
#define WLP_ID_ARCHIVE        3
#define WLP_ID_HEARTBEAT      4
#define WLP_ID_SET_MODE       5
#define WLP_ID_SET_CHANNEL    6
#define WLP_ID_CONF_PS        7
#define WLP_ID_SET_PS         8
#define WLP_ID_SCAN           9
#define WLP_ID_SET_WPS        10
#define WLP_ID_CONF_AP        11
#define WLP_ID_LINKUP         12
#define WLP_ID_LINKDOWN       13
#define WLP_ID_GET_LINKCTX    14
#define WLP_ID_SET_LINKCTX    15
#define WLP_ID_DATA           16
#define WLP_ID_START_TX       17
#define WLP_ID_STOP_TX        18
#define WLP_ID_GET_RXSTATS    19
#define WLP_ID_CLR_RXSTATS    20
#if WLP_CONFIG_STACK_OFFLOAD
# define WLP_ID_SET_IPADDR     32
# define WLP_ID_SET_DHCP       33
# define WLP_ID_SOCKET         34
# define WLP_ID_BIND           35
# define WLP_ID_LISTEN         36
# define WLP_ID_CONNECT        37
# define WLP_ID_ACCEPT         38
# define WLP_ID_CLOSE          39
# define WLP_ID_RECVED         40
# define WLP_ID_SENT           41
# define WLP_ID_SEND           42
# define WLP_ID_SENDTO         43
# define WLP_ID_RECV           44
# define WLP_ID_GET_HOSTBYNAME 45
# define WLP_ID_SET_DHCPD      46
#endif /* WLP_CONFIG_STACK_OFFLOAD */
#define WLP_ID_ADD_MCAST      21
#define WLP_ID_DEL_MCAST      22
#define WLP_ID_SET_BAUDRATE   23

#define WLP_ID_MASK           0xfff
#define WLP_ID(id) (id & WLP_ID_MASK)

/* Message alignment */
#define WLP_MSG_ALIGNMENT 4
#define WLP_MSG_MIN_SIZE  32
#define WLP_MSG_ALIGNMENT_MASK (WLP_MSG_ALIGNMENT - 1)

/* Pad to WLP_MSG_MIN_SIZE or to align to WLP_MSG_ALIGNMENT */
#define wlp_pad(len)                                                    \
        ((len) < WLP_MSG_MIN_SIZE ? WLP_MSG_MIN_SIZE - (len) :          \
         (((len) & WLP_MSG_ALIGNMENT_MASK) ?                            \
          WLP_MSG_ALIGNMENT - ((len) & WLP_MSG_ALIGNMENT_MASK) : 0))

#include <owl_beginpack.h>

struct wlp_init_req {
#define HDG205_STACK_OFFLOAD      (1 << 0)
#define HDG205_SDIO               (1 << 1)
        uint8_t flags;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_loopback_req {
        uint16_t count;
        uint16_t len;
        uint8_t data[0];
} OWL_PACKED;

struct wlp_archive_req {
        uint16_t len;
#define HDG205_ARCHIVE_MORE       (1 << 0)
        uint16_t flags;
        uint8_t data[0];
} OWL_PACKED;

struct wlp_set_mode_req {
        uint8_t mode;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_set_channel_req {
        uint8_t channel;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_set_baudrate_req {
        uint32_t baudrate;
} OWL_PACKED;

struct wlp_conf_ps_req {
        uint32_t ps_delay;
        uint32_t traffic_tmo;
        uint16_t listen_intvl;
        uint8_t ps_poll;
        uint8_t rx_all_dtim;
} OWL_PACKED;

struct wlp_set_ps_req {
        uint8_t enable;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_set_wps_req {
        uint8_t enable;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_conf_ap_req {
        uint16_t inact_tmo;
        uint16_t beacon_intvl;
        uint8_t hidden;
        char country[7];
} OWL_PACKED;

struct wlp_scan_req {
        struct wlp_ssid ssid;
} OWL_PACKED;

struct wlp_linkup_req {
        struct wlp_ssid ssid;
        char pad1[3];
        char key[WLP_MAX_PASS_LEN];
        uint8_t auth;
        char pad2[3];
} OWL_PACKED;

struct wlp_set_linkctx_req {
        uint16_t len;
        char pad[2];
        uint8_t data[0];
} OWL_PACKED;

struct wlp_data_req {
        uint16_t vlanid_prio;
        uint8_t pad[2];
        uint8_t data[0];
} OWL_PACKED;

struct wlp_start_tx_req {
        uint32_t count;
        uint16_t intvl;
        uint16_t size;
        uint8_t rate;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_get_rxstats_req {
        uint8_t rate;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_add_mcast_req {
        struct wlp_mac_addr hwaddr;
#define HDG205_MCAST_ALL      (1 << 0)
        uint8_t flags;
        uint8_t pad[1];
} OWL_PACKED;

struct wlp_del_mcast_req {
        struct wlp_mac_addr hwaddr;
#define HDG205_MCAST_ALL      (1 << 0)
        uint8_t flags;
        uint8_t pad[1];
} OWL_PACKED;

struct wlp_data_ind {
        /* The highest bit in vlanid_prio is used to indicate whether
           we have an updated SNR value in the rssi_snr field. If '1',
           then the field rssi_snr contains a new snr value, otherwise
           it contains a rssi value. */
#define HDG205_SNR_UPDATED (1 << 15)
        uint16_t vlanid_prio;
        int8_t rssi_snr;
        uint8_t rate;
        uint8_t data[0];
} OWL_PACKED;

struct wlp_get_rxstats_ind {
        struct wlp_rfstats rxstats;
} OWL_PACKED;

struct wlp_get_linkctx_ind {
        uint16_t len;
        uint8_t data[0];
} OWL_PACKED;

struct wlp_scan_ind {
        struct wlp_network net;
} OWL_PACKED;

struct wlp_linkup_ind {
        struct wlp_network net;
} OWL_PACKED;

struct wlp_init_cfm {
        
        /* Must be at offset:0 (1.1.2/1.1.4) */
        struct wlp_mac_addr hwaddr;

        /* Unused (1.1.2/1.1.4) */
        uint8_t pad[2];
        uint16_t caps;

        /* Must be at offset:10 (1.1.2/1.1.4) */
        uint16_t bufcnt;   /* Num rx buffers */
        uint16_t buflen;   /* RX buffer size */

        /* Unused (1.1.2/1.1.4) */
        uint8_t pad2[2];
        uint16_t board;

        /* Must be at offset:18 (1.1.2/1.1.4).
           The size of this field was changed from 8 to 16 in 1.1.6/1.1.9 */
        char version[16];

        /* Unused (1.1.2/1.1.4) */
        char build[16];
        char date[20];
        char target[32];

} OWL_PACKED;

struct wlp_data_cfm {
        int8_t rssi;
        uint8_t rate;
        uint8_t pad[2];
} OWL_PACKED;

struct wlp_loopback_cfm {
        uint16_t len;
        uint8_t data[0];
} OWL_PACKED;


#if WLP_CONFIG_STACK_OFFLOAD
struct wlp_set_ipaddr_req {
        uint32_t ip;
        uint32_t netmask;
        uint32_t gw;
        uint32_t dns;
} OWL_PACKED;

struct wlp_set_dhcp_req {
        uint8_t enable;
        uint8_t pad[3];
} OWL_PACKED;
                
struct wlp_socket_req {
        int16_t xid;
        uint8_t type;
        uint8_t proto;
} OWL_PACKED; 

struct wlp_bind_req {
        uint32_t ip;
        int16_t xid;
        uint16_t port;
} OWL_PACKED;

struct wlp_listen_req {
        int16_t xid;
        int8_t backlog;
        uint8_t pad[1];
} OWL_PACKED;

struct wlp_connect_req {
        uint32_t ip;
        int16_t xid;
        uint16_t port;
} OWL_PACKED;

struct wlp_accept_req {
        int16_t xid;
        int16_t pad[2];
} OWL_PACKED;

struct wlp_close_req {
        int16_t xid;
        uint8_t pad[2];
} OWL_PACKED;

struct wlp_recved_req {
        int16_t xid;
        int16_t len;
} OWL_PACKED;

struct wlp_send_req {
        int16_t xid;
        int16_t len;
        uint8_t data[0];
} OWL_PACKED;

struct wlp_sendto_req {
        uint32_t ip;
        uint16_t port;
        int16_t xid;
        int16_t len;
        uint8_t data[0];
} OWL_PACKED;

struct wlp_get_hostbyname_req {
#define WLP_HOSTNAME_MAX_LENGTH 127
        char host[WLP_HOSTNAME_MAX_LENGTH + 1];        
} OWL_PACKED;

struct wlp_set_dhcpd_req {
        uint8_t enable;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_set_ipaddr_ind {
        uint32_t ip;
        uint32_t netmask;
        uint32_t gw;
        uint32_t dns;
} OWL_PACKED;

struct wlp_get_hostbyname_ind {
        uint32_t ip;
} OWL_PACKED;

struct wlp_connect_ind {
        int16_t xid;
        uint16_t port;
        uint32_t ip;
        int8_t connected;
        uint8_t pad[3];
} OWL_PACKED;

struct wlp_listen_ind {
        int16_t lxid;
        int16_t xid;
        uint32_t ip;
        uint16_t port;
        uint8_t pad[2];
} OWL_PACKED;

struct wlp_recv_ind {
        int16_t xid;
        int16_t len;
        uint32_t ip;
        uint16_t port;
        uint8_t pad[2];
        
        uint8_t data[0];
    
} OWL_PACKED;

struct wlp_sent_ind {
        int16_t xid;
        int16_t len;
} OWL_PACKED;

#endif /* WLP_CONFIG_STACK_OFFLOAD */


struct wlp_hdr {
#if WLP_HPI_HDR_SIZE
        uint8_t hpi[WLP_HPI_HDR_SIZE];
#endif /* WLP_HPI_HDR_SIZE */
        uint16_t len; /* Length of data, NOT including header. */
        uint16_t id;
        uint8_t pad;  /* The pad is also included in the len field */
        int8_t res;
} OWL_PACKED;

struct wlp_req {
        union {
                struct wlp_init_req init;
                struct wlp_loopback_req loopback;
                struct wlp_archive_req archive;
            
                struct wlp_set_mode_req set_mode;
                struct wlp_set_channel_req set_channel;
                struct wlp_conf_ps_req conf_ps;
                struct wlp_set_ps_req set_ps;
                
                struct wlp_linkup_req linkup;
                struct wlp_scan_req scan;
                struct wlp_set_wps_req set_wps;
                struct wlp_conf_ap_req conf_ap;

                struct wlp_data_req data;
                
                struct wlp_set_linkctx_req set_linkctx;
                struct wlp_start_tx_req start_tx;
                struct wlp_get_rxstats_req get_rxstats;

                struct wlp_add_mcast_req add_mcast;
                struct wlp_del_mcast_req del_mcast;

                struct wlp_set_baudrate_req set_baudrate;

#if WLP_CONFIG_STACK_OFFLOAD
                struct wlp_set_ipaddr_req set_ipaddr;
                struct wlp_set_dhcp_req set_dhcp;
                
                struct wlp_socket_req socket;
                struct wlp_bind_req bind;
                struct wlp_listen_req listen;
                struct wlp_connect_req connect;
                struct wlp_accept_req accept;
                struct wlp_close_req close;
                struct wlp_recved_req recved;
                struct wlp_send_req send;
                struct wlp_sendto_req sendto;
                struct wlp_get_hostbyname_req get_hostbyname;

                struct wlp_set_dhcpd_req set_dhcpd;
#endif /* WLP_CONFIG_STACK_OFFLOAD */
                
        } u; /* C89 */
} OWL_PACKED;

struct wlp_cfm {
        union {
                
                struct wlp_init_cfm init;
                struct wlp_loopback_cfm loopback;
                struct wlp_data_cfm data;
        } u; /* C89 */
} OWL_PACKED;

struct wlp_ind {
        union {
                struct wlp_linkup_ind linkup;
                struct wlp_scan_ind scan;
                struct wlp_data_ind data;
                struct wlp_get_linkctx_ind get_linkctx;
                struct wlp_get_rxstats_ind get_rxstats;
                
#if WLP_CONFIG_STACK_OFFLOAD
                struct wlp_set_ipaddr_ind set_ipaddr;
                struct wlp_get_hostbyname_ind get_hostbyname;
                struct wlp_connect_ind connect;
                struct wlp_listen_ind listen;
                struct wlp_recv_ind recv;
                struct wlp_sent_ind sent;
#endif /* WLP_CONFIG_STACK_OFFLOAD */
        } u;
} OWL_PACKED;

struct wlp_msg {
        struct wlp_hdr hdr;
        union {
                struct wlp_req req;
                struct wlp_cfm cfm;
                struct wlp_ind ind;
        } u; /* C89 */
} OWL_PACKED;

#include <owl_endpack.h>

#endif /* WLP_PROTO_H */
