/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_CONCURRENT_H
#define WLP_CONCURRENT_H

#if defined(WLP_CONFIG_TRAMPOLINE)

int _wlp_get_iface(void);
int _wlp_set_iface(int iface);

#else  /* defined(WLP_CONFIG_TRAMPOLINE) */

# define wlp_get_iface           _wlp_get_iface
# define wlp_set_iface           _wlp_set_iface

#endif /* defined(WLP_CONFIG_TRAMPOLINE) */


/*! \defgroup wlp_iface Concurrent Interfaces Funcations
 *
 * These functions manage Wi-Fi device when concurrent mode is supported.
 *
 * @{
 *
 */ 



/**
 * \brief Get the current Wi-Fi interface.
 *
 * @returns the interface set by wlp_set_iface().
 */
int wlp_get_iface(void);

/**
 * \brief Set the current Wi-Fi interface.
 *
 * For devices that support concurrent operation, the interface selected with
 * wlp_set_iface() will be used in future calls to other functions in the
 * wlp_api. Therefore it is important to ensure that the proper interface
 * is selected before any other function is called.
 *
 * See the documentation for respective function in the wlp_api to see how it is
 * affected by wlp_set_iface().
 *
 * @returns the previous interface instance.
 *
 *
 */
int wlp_set_iface(int iface);

/*! @} */

#endif /* WLP_CONCURRENT_H */
