/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_INET_H
#define WLP_INET_H

#include <owl_cc.h>

#include <wlp_ipaddr.h>

#ifndef LITTLE_ENDIAN
# define LITTLE_ENDIAN 0x0a0b0c0d /* LSB first */
#endif

#ifndef BIG_ENDIAN
# define BIG_ENDIAN    0x0d0c0b0a /* MSB first */
#endif

#if !defined(WLP_CONFIG_LWIP)

# if BYTE_ORDER == LITTLE_ENDIAN
uint16_t htons(uint16_t x);
uint16_t ntohs(uint16_t x);
uint32_t htonl(uint32_t x);
uint32_t ntohl(uint32_t x);
# else /* BYTE_ORDER == LITTLE_ENDIAN */
#  define htons(x) (x)
#  define ntohs(x) (x)
#  define htonl(x) (x)
#  define ntohl(x) (x)
# endif /* BYTE_ORDER == LITTLE_ENDIAN */

int ip_aton(const char *cp, struct ip_addr *addr);
char *ip_ntoa(const struct ip_addr *addr);

#else /* !defined(WLP_CONFIG_LWIP) */

# include <lwip/ip_addr.h>
# ifndef ip_ntoa /* defined by lwip-1.3.2+ but not by 1.3.0 */
#  define ip_ntoa(addr) inet_ntoa(*((struct in_addr*)(addr)))
# endif /* ip_ntoa */
# define ip_aton ipaddr_aton

# endif /* !defined(WLP_CONFIG_LWIP) */

#endif /* WLP_INET_H */
