/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */


#ifndef WLP_IPADDR_H
#define WLP_IPADDR_H

#include <owl_cc.h>

/**
 * @file wlp_ipaddr.h
 *  
 * @author H&D Wireless AB
 * @brief oWL Pico API
 *
 */


/*
 * The struct ip_addr type is based on work from the lwIP network stack.
 *
 */

/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification,  are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#if !defined(WLP_CONFIG_LWIP)

struct ip_addr {
        uint32_t addr;
};

extern const struct ip_addr ip_addr_any;

#define IP_ADDR_ANY ((struct ip_addr *) &ip_addr_any)
#define IP4_ADDR(ipaddr, a,b,c,d)                               \
        (ipaddr)->addr = htonl(((uint32_t)((a) & 0xff) << 24) | \
                               ((uint32_t)((b) & 0xff) << 16) | \
                               ((uint32_t)((c) & 0xff) << 8) |  \
                               (uint32_t)((d) & 0xff))

#define ip_addr_set(dest, src)                          \
        (dest)->addr = ((src) == NULL? 0: (src)->addr)
#define ip_addr_netcmp(addr1, addr2, mask) \
        (((addr1)->addr & (mask)->addr) == ((addr2)->addr & (mask)->addr))
#define ip_addr_cmp(addr1, addr2) \
        ((addr1)->addr == (addr2)->addr)
#define ip_addr_isany(addr1) \
        ((addr1) == NULL || (addr1)->addr == 0)

#define ip4_addr1(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr) >> 24) & 0xff)
#define ip4_addr2(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr) >> 16) & 0xff)
#define ip4_addr3(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr) >> 8) & 0xff)
#define ip4_addr4(ipaddr) ((uint16_t)(ntohl((ipaddr)->addr)) & 0xff)

#define ip4_addr_set_u32(dest_ipaddr, src_u32)  \
        ((dest_ipaddr)->addr = (src_u32))
#define ip4_addr_get_u32(src_ipaddr) ((src_ipaddr)->addr)

#else

# include <lwip/ip_addr.h>

#endif /* !defined(WLP_CONFIG_LWIP) */

#define IP4_INIT(a, b, c, d)                               \
        ((uint32_t) ((d) & 0xff) << 24) |                  \
        ((uint32_t) ((c) & 0xff) << 16) |                  \
        ((uint32_t) ((b) & 0xff) << 8)  |                  \
        (uint32_t) ((a) & 0xff)


#if defined(WLP_CONFIG_TRAMPOLINE)

void _wlp_set_ipaddr_cb(void (*addr_cb)(void *ctx, const struct ip_addr *addr),
                       void *ctx);
int _wlp_set_ipaddr(const struct ip_addr *ip, const struct ip_addr *netmask,
                   const struct ip_addr *gw, const struct ip_addr *dns);
int _wlp_get_ipaddr(struct ip_addr *ip, struct ip_addr *netmask,
                   struct ip_addr *gw, struct ip_addr *dns);
int _wlp_set_dhcp(int enable);
int _wlp_get_dhcp(int *enabled);

#else /* defined(WLP_CONFIG_TRAMPOLINE) */

#define wlp_set_ipaddr_cb       _wlp_set_ipaddr_cb
#define wlp_set_ipaddr          _wlp_set_ipaddr
#define wlp_get_ipaddr          _wlp_get_ipaddr
#define wlp_set_dhcp            _wlp_set_dhcp
#define wlp_get_dhcp            _wlp_get_dhcp

#endif /* defined(WLP_CONFIG_TRAMPOLINE) */


/*! @} */

/*! \defgroup wlp_network Network address functions
 *
 * These functions manage the network address configuration.
 *
 * @{
 *
 */ 

/**
 * \brief Register a callback that will be invoked when an IP address is set.
 *
 * The provided callback will be invoked when an IP address is set or removed.
 * Every time wlp_poll() is invoked, the callback condition will be evaluated
 * and any registered callback will be invoked from same context as the call to
 * wlp_poll().
 *
 * If a static IP address is configured (through wlp_set_ipaddr()), this
 * function will be called at the same time as the link callback registered in
 * wlp_set_link_cb().
 *
 * If a dynamic IP address is configured (through wlp_set_dhcp()) this function
 * will be called when the link is up and an address has been successfully
 * obatined through DHCP. It will also be called when the wifi link goes down
 * (since the IP address will be removed at that time) or if the IP
 * address is relesed through a call to set_dhcp() with argument 0.
 *
 * Note that the connection status will also be indicated by any wlp_api that
 * performs socket communication (e.g. wlp_send() and wlp_recv()) through
 * their return values, even though wlp_set_ipaddr_cb() has not been called.
 *
 * @param addr_cb is called when the IP address is set or removed. If the
 *              address is set, the current address will be provided in the addr
 *              parameter. If the address is removed, the addr parameter will
 *              be NULL.
 * @param ctx is an opaque context parameter that will be passed back to the
 *              addr_cb.
 *
 */
void wlp_set_ipaddr_cb(void (*addr_cb)(void *ctx, const struct ip_addr *addr),
                       void *ctx);


/**
 * \brief Set a static IP address
 *
 * Disable DHCP and configure the device static IP address settings.
 * wlp_set_ipaddr() can be invoked either before or after the wifi link is up.
 *
 * If a static IP address is configured using this function, any callback
 * registered with wlp_set_ipaddr_cb() will be invoked as soon as the
 * wifi link status changes (see wlp_linkup()).
 *
 * @param ip is the IP address to set, e.g. 192.168.1.100. Not used if NULL.
 * @param mask is the network mask, e.g. 255.255.255.0. Not used if NULL.
 * @param gw is the gateway to use, e.g. 192.168.1.1. Not used if NULL.
 * @param dns is the dns server to use, e.g. 192.168.1.1. Not used if NULL.
 *
 * @return
 *   - 0 on success.
 *
 */
int wlp_set_ipaddr(const struct ip_addr *ip, const struct ip_addr *netmask,
                   const struct ip_addr *gw, const struct ip_addr *dns);


/**
 * \brief Get the current IP address parameters
 *
 * The currently assigned IP address setttings for the wifi device will be
 * fetched. If DHCP is enabled (through wlp_set_dhcp()) and the IP configuration
 * has not yet been assigned, all IP address values will be IP_ADDR_ANY.
 * If DHCP is disabled the statically assigned IP addresses will be fetched.
 *
 * @param ip will hold the current IP address upon successful return. Not used
 *              if NULL.
 * @param netmask will hold the current netmask upon successful return. Not used
 *              if NULL.
 * @param gw will hold the current default gateway address upon successful
 *              return. Not used if NULL.
 * @param dns will hold the current dns server address upon successful return.
 *              return. Not used if NULL.
 *
 * @return
 *   - 0 on success
 *   - WLP_ERR_CONN if not connected (IP address configuration will still
 *              be filled in into the provided parameters).
 *
 */
int wlp_get_ipaddr(struct ip_addr *ip, struct ip_addr *netmask,
                   struct ip_addr *gw, struct ip_addr *dns);


/**
 * \brief Request or release IP address through DHCP.
 *
 * Enable or disable DHCP for IP address configuration. If this function
 * is called when the wifi link is up, an IP address will be requested or
 * released upon successful return, depending on the value of the enable
 * parameter. See wlp_set_ipaddr_cb() on how to get notified when the IP address
 * configuration is set or removed.
 *
 * wlp_set_dhcp() can be invoked either before or after the wifi link is up.
 * If this function is called when the wifi link is down, any DHCP requests
 * will be performed when the wifi link comes up.
 *
 * If DHCP is enabled through this function, any configuration set through
 * wl_set_ipaddr() will not be used.
 * 
 * Note that the current IP address settings can be obtained through
 * wl_get_ipaddr() at any time.
 *
 * @param enable determines if DHCP should be enabled or disabled.
 *
 * @return 0 on success.
 *
 */
int wlp_set_dhcp(int enable);


/**
 * \brief Get the current DHCP configuration
 *
 * Get information on whether DHCP is enabled or not. Note that wlp_get_dhcp()
 * will not indicate whether an actual IP address has been obtained through DHCP
 * or not. See wlp_set_ipaddr_cb() for information about IP address
 * notifications.
 *
 * @param enabled will hold the DHCP configuration upon successful return. Not
 *              used if NULL.
 * @return
 *   - 0 on success
 *   - WLP_ERR_CONN if not connected (DHCP configuration will still be
 *              filled in into the provided parameter).
 *
 */
int wlp_get_dhcp(int *enabled);

    
/*! @} */

#endif /* WLP_IPADDR_H */
