/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */

#ifndef WLP_LINK_H
#define WLP_LINK_H

#include <owl_cc.h>

/*! \ingroup wlp_link */
enum wlp_auth_mode {
        WLP_AUTH_AUTO = 0,
        WLP_AUTH_WEP,
        WLP_AUTH_WPA_PSK,
        WLP_AUTH_WPA2_PSK,
        WLP_AUTH_WPA,
        WLP_AUTH_WPA2,
};

#define WLP_MAX_PASS_LEN 64
#define WLP_SNR_UNKNOWN -128
#define WLP_RSSI_UNKNOWN -128
#define WLP_SSID_MAX_LENGTH 32
#define WLP_MAC_ADDR_LENGTH 6

#define WLP_MODE_STA    0
#define WLP_MODE_AP     1
#define WLP_MODE_ADHOC  2
#define WLP_MODE_RFTEST 3

#define WLP_RATE_1MBIT      0x00   /* QPSK */
#define WLP_RATE_2MBIT      0x01   /* QPSK */
#define WLP_RATE_5_5MBIT    0x02   /* QPSK */
#define WLP_RATE_11MBIT     0x03   /* QPSK */
#define WLP_RATE_6MBIT      0x10   /* OFDM */
#define WLP_RATE_9MBIT      0x11   /* OFDM */
#define WLP_RATE_12MBIT     0x12   /* OFDM */
#define WLP_RATE_18MBIT     0x13   /* OFDM */
#define WLP_RATE_24MBIT     0x14   /* OFDM */
#define WLP_RATE_36MBIT     0x15   /* OFDM */
#define WLP_RATE_48MBIT     0x16   /* OFDM */
#define WLP_RATE_54MBIT     0x17   /* OFDM */
#define WLP_RATE_6_5MBIT    0x20   /* HT MCS */
#define WLP_RATE_13MBIT     0x21   /* HT MCS */
#define WLP_RATE_19_5MBIT   0x22   /* HT MCS */
#define WLP_RATE_26MBIT     0x23   /* HT MCS */
#define WLP_RATE_39MBIT     0x24   /* HT MCS */
#define WLP_RATE_52MBIT     0x25   /* HT MCS */
#define WLP_RATE_58_5MBIT   0x26   /* HT MCS */
#define WLP_RATE_65MBIT     0x27   /* HT MCS */
#define WLP_RATE_UNKNOWN    0xff

#include <owl_beginpack.h>

/*! \ingroup wlp_link
 *
 * \brief SSID representation.
 *
 * The SSID is a binary string and cannot be treated as a
 * C-string safely. An empty SSID is represented by a
 * SSID struct with the len field set to 0.
 */
struct wlp_ssid {
        char ssid[WLP_SSID_MAX_LENGTH];
        uint8_t len;
} OWL_PACKED;


/*! \ingroup wlp_link
 *
 * MAC-address/BSSID representation 
 *
 * A broadcast BSSID is one with all octets set to 0xFF.
 */
struct wlp_mac_addr {
        uint8_t octet[WLP_MAC_ADDR_LENGTH];
} OWL_PACKED;

/*! \ingroup wlp_link
 * Wireless network information structure.
 *
 *
 */
struct wlp_network {
        struct wlp_ssid ssid;          /*!< The SSID of the network */
        uint8_t auth;
        uint8_t channel;
#define WLP_NET_FLAG_ADHOC (1 << 0)
        uint8_t flags;

        struct wlp_mac_addr bssid;     /*!< The BSSID of the network */
        int16_t rssi;                   /**< Received Signal Strength in dBm
                                           (measured on beacons) */
        int16_t snr;                    /**< Received Signal to noise ratio in
                                           dBm (measured on beacons) */
} OWL_PACKED;

#include <owl_endpack.h>


/* WLP_CONFIG_TRAMPOLINE can be used to wrap the wlp_api without changing the
   API. This is useful e.g. if an RTOS layer is added. If unsure, then set
   WLP_CONFIG_TRAMPOLINE to 0. */
#if defined(WLP_CONFIG_TRAMPOLINE)

int _wlp_scan(struct wlp_ssid *ssid,
              void (*scan_cb)(void *ctx, struct wlp_network *net), void *ctx);
void _wlp_set_link_cb(void (*link_cb)(void *ctx, int link), void *ctx);
int _wlp_linkup(struct wlp_ssid *ssid, const char *key,
                enum wlp_auth_mode auth);
int _wlp_linkdown(void);
int _wlp_set_wps(int enable);
int _wlp_get_hwaddr(struct wlp_mac_addr *hwaddr);
int _wlp_get_network(struct wlp_network *net);
int _wlp_get_rate(uint8_t *tx, uint8_t *rx);
int _wlp_get_link_context(void (*linkctx_cb)(void *ctx, const void *linkctx, int len),
                          void *ctx);
int _wlp_set_link_context(const void *linkctx, int len);
int _wlp_set_mode(uint8_t mode);
int _wlp_get_mode(uint8_t *mode);
int _wlp_set_channel(uint8_t channel,
                     void (*channel_cb)(void *ctx, int status), void *ctx);
int _wlp_conf_ap(int hidden, int beacon_intvl, int inact_tmo,
                const char *country);
int _wlp_set_ps(int enable);
int _wlp_conf_ps(uint8_t use_ps_poll,
                uint32_t ps_traffic_timeout,
                uint32_t ps_delay,
                uint8_t rx_all_dtim,
                uint16_t listen_interval);

#else /* defined(WLP_CONFIG_TRAMPOLINE) */

# define wlp_scan                _wlp_scan
# define wlp_set_link_cb         _wlp_set_link_cb
# define wlp_linkup              _wlp_linkup
# define wlp_linkdown            _wlp_linkdown
# define wlp_set_wps             _wlp_set_wps
# define wlp_get_hwaddr          _wlp_get_hwaddr
# define wlp_get_network         _wlp_get_network
# define wlp_get_rate            _wlp_get_rate
# define wlp_get_link_context    _wlp_get_link_context
# define wlp_set_link_context    _wlp_set_link_context
# define wlp_set_mode            _wlp_set_mode
# define wlp_get_mode            _wlp_get_mode
# define wlp_set_channel         _wlp_set_channel
# define wlp_conf_ap             _wlp_conf_ap
# define wlp_conf_ps             _wlp_conf_ps
# define wlp_set_ps              _wlp_set_ps

#endif /* defined(WLP_CONFIG_TRAMPOLINE) */


/*! \defgroup wlp_link Wi-Fi link management functions
 *
 * These functions manage the Wi-Fi connection such as encryption
 * keys, MAC addresses, and connecting to access points.
 *
 * @{
 *
 */ 


/**
 * \brief Start a scan on all channels.
 *
 * The scan results will be passed \a scan_cb one at a time.
 *
 * Note : Scanning may interfere with the connection management on
 *        some WiFi modules. If the module is already scanning because
 *        it is trying to connect to a network then wlp_scan() may
 *        return WLP_ERR_BUSY. To get manual scanning to work in
 *        that situation you may have to issue a wlp_linkdown() call
 *        first to stop the connection manager from scanning.
 *
 * @param ssid SSID to scan for. Can be NULL, in which case all SSIDs
 *       are included in the result.
 * @param scan_cb Callback that will be invoked once for every network
 *       found. The net parameter will be NULL when the scan is completed.
 * @param ctx is an opaque context pointer that will be passed back to 
 *        scan_cb.
 */
int wlp_scan(struct wlp_ssid *ssid,
             void (*scan_cb)(void *ctx, struct wlp_network *net), void *ctx);

/**
 * \brief Set a wifi link status callback function.
 *
 * Register a callback that will be invoked when the wifi connection status
 * changes.
 *
 * Every time wlp_poll() is invoked, the callback condition will be evaluated
 * and any registered callback will be invoked from same context as the call to
 * wlp_poll().
 *
 * Note that the link callback will be invoked when the wifi device is
 * connected to the access point, an IP address does not have to be
 * configured. See wlp_set_ipaddr_cb() for information on how to get notified 
 * when an IP address is configured.
 *
 * Note that the connection status will also be indicated by any wlp_api that
 * performs socket communication (e.g. wlp_send() and wlp_recv()) through
 * their return values, event though wlp_set_link_cb() is not called.
 *
 * In access point mode, the link callback will be invoked when the first
 * client is connected to the access point. See wlp_set_mode() for more
 * information on how to set access point mode.
 *
 * @param link_cb will be invoked upon a link status change. The callback
 *              parameter \a link will be set to 1 to indicate that the link
 *              has been established and 0 to indicate that the link has been
 *              shut down. If a connection error occured, \a link will be set
 *              to an error code indicating the type of failure that occured.
 *              Note that the driver will retry the connection after connection
                errors, unless wlp_linkdown() has been called.
 * @param ctx is an opaque context pointer that will be passed back to link_cb.
 *
 */
void wlp_set_link_cb(void (*link_cb)(void *ctx, int link), void *ctx);


/**
 * \brief Connect the wifi link.
 *
 * Enable the wifi link. Upon successful return, the wifi device will try
 * to establish and maintain a connection to the network specified by the
 * parameters. When the link status (connected, disconnected) changes, the
 * caller will be notified of the new status through the callback registered
 * in wlp_set_link_cb(). Information about the current network can be obtained
 * through wlp_get_network().
 *
 * Note again that successful return means that the wifi device will now
 * _initate_ a link using the desired parameters, not that the link
 * is already established.
 *
 * If wlp_linkup() is called when already connected to an access point, the
 * wifi device will try to switch to the network configuration according to
 * the provided parameters.
 *
 * wlp_linkup() will only manage the link layer, not the network or socket
 * layer. See wlp_set_dhcp() and wlp_set_ipaddr() for information about
 * configuring the network layer and wlp_socket() for information about the
 * socket layer.
 *
 * In access point mode, this function will start an access point with the
 * given ssid and key parameters. See wlp_set_mode() for more information on
 * how to set access point mode.
 *
 * If the \a auth parameter is WLP_AUTH_AUTO WPA2 authentication with CCMP
 * encryption will be used for security (in access point).
 *
 * @param ssid is the name of the access point to connect to or the name
 *              of the access point to start (in access point mode).

 * @param key is the encryption key to use. If this parameter is NULL,
 *              no enryption will be used. Should be provided as an
 *              ascii string. In case of a WPA key (parameter auth set
 *              to WLP_AUTH_AUTO, WLP_AUTH_WPA_PSK or
 *              WLP_AUTH_WPA2_PSK), the WPA passphrase should be
 *              provided, e.g. "mysecretkey".  A WPA pass-phase must
 *              be between 8 and 63 characters long.  In case of a WEP
 *              key (parameter auth set to WLP_AUTH_WEP), the key
 *              should be provided as a string containing the
 *              hexadecimal chars (10 or 26 chars, depending on
 *              whether a 40-bit or 104-bit key is used),
 *              e.g. "a1b2c3d4e5".

 * @param auth specifies the authentication mode to use. For WEP this
 *             parameter must be set to WLP_AUTH_WEP. For WPA/WPA2 the
 *             parameter can be set to WLP_AUTH_AUTO to select the
 *             mode from the network, or to WLP_AUTH_WPA_PSK or
 *             WLP_AUTH_WPA2_PSK to explicitly select WPA or
 *             WPA2/RSN. WLP_AUTH_AUTO also allows open authentication
 *             (no encryption) and for security reasons should not be
 *             used when the actual authentication mode of the network
 *             is known.
 *
 * @return
 *   - 0 on success
 *   - WLP_ERR_PARAM if invalid parameters, e.g. invalid key length.
 *   - WLP_ERR_FAILURE on any other error.
 *
 */
int wlp_linkup(struct wlp_ssid *ssid, const char *key, enum wlp_auth_mode auth);


/**
 * \brief Disconnect the wifi link.
 * 
 * Disable the wifi link. The wifi device will no longer try to maintain the
 * network connection. Any link callback that was registered through
 * wlp_set_link_cb() will be invoked when disconnection is completed,
 * if currently connected to an access point.
 *
 * Note again that successful return means that the wifi device will now
 * _initate_ a link disconnection, not that the link is already disconnected.
 *
 * If wlp_linkup() is called when already disconnected, any ongoing connection
 * attempts will be stopped.
 *
 * wlp_linkdown() will only manage the link layer, not the network or socket
 * layer. See wlp_set_dhcp() and wlp_set_ipaddr() for information about
 * configuring the network layer and wlp_socket() for information about the
 * socket layer.
 *
 * In access point mode, this function will stop the access point and it will
 * no longer transmit beacons. All clients will be disconnected. See
 * wlp_set_mode() for more information on how to set access point mode.
 *
 * @return
 *   - 0 on success
 *
 */
int wlp_linkdown(void);


/**
 * \brief Enable or disable WPS (Wi-Fi Protected Setup).
 *
 * When enabled, pin or push button methods can be used to connect to an
 * access-point. To use the pin method, the pin should be passed to
 * wlp_linkup() after enabling WPS. To use the push-button method, wlp_linkup()
 * should be called without a key after enabling WPS.
 *
 * @param enable determines if WPS should be enabled or disabled.
 *
 * @return 0 on success.
 *
 */
int wlp_set_wps(int enable);


/**
 * \brief Get the interface MAC address.
 *
 * Return the 802.3 MAC address of the network interface.
 * 
 * @param hwaddr will hold the wifi device MAC address upon successful return.
 * @return
 *   - 0 on success
 *   - A negative error code on failure
 *
 */
int wlp_get_hwaddr(struct wlp_mac_addr *hwaddr);


/**
 * \brief Get the 802.11 network.
 * 
 * Get the description of the currently associated network, if any.
 *
 * If associated, the device will update the RSSI and SNR information each
 * time this function is called by listening for beacon frames from the
 * current AP.
 *
 * In access point mode, this function will return the description of the
 * device when the link is up, e.g. first client connected. See wlp_set_mode()
 * for more information on how to set access point mode. In this mode, the
 * RSSI and SNR values will be set to 0.
 *
 * 
 *
 * @param net will hold the network description upon successful return.
 * @return
 *   - 0 on success
 *   - A negative error code on failure
 *
 */
int wlp_get_network(struct wlp_network *net);


/**
 * \brief Get current TX and RX rate used for data transfer
 *
 * During transmission and reception of data, the actual rate used will depend
 * on the signal quality. This function can be used to get the actual rate used
 * for the last tx and rx data.
 *
 * @param tx will hold the tx rate upon successful return.
 * @param rx will hold the rx rate upon successful return.
 *
 * @return
 *   - 0 on success
 *   - A negative error code on failure.
 */
int wlp_get_rate(uint8_t *tx, uint8_t *rx);


/**
 * Retrieve information about the link, such as the PMK. This can be used to
 * later speed up the linkup process by passing the same context to
 * wlx_set_link_context().
 *
 */
int wlp_get_link_context(void (*linkctx_cb)(void *ctx, const void *linkctx,
                                            int len),
                         void *ctx);


/**
 * Set context for the link. The context is only valid for the next
 * wlp_linkup() call.
 *
 */
int wlp_set_link_context(const void *linkctx, int len);


/*! @} */



/*! \defgroup wlp_ap Access point functions
 *
 * These functions manage the access point mode.
 *
 * @{
 *
 */ 


/**
 * \brief Set the Wi-Fi mode to AP or STA.
 *
 * wlp_set_mode() will configure the Wi-Fi device to operate either as a
 * station or as an access point. The current mode can be obtained through
 * wlp_get_mode().
 *
 * @param mode is the mode to configure. This must be either WLP_MODE_STA,
 *              WLP_MODE_ADHOC or WLP_MODE_AP.
 *
 * @return
 *   - 0 on success
 *   
 */
int wlp_set_mode(uint8_t mode);



/**
 * \brief Check if the device is currently in AP or STA mode.
 *
 * wlp_get_mode() will be return the current operation mode of the Wi-Fi device.
 * Since the operation mode is persistent between power-cycles, it is generally
 * a good idea to call this function at startup to ensure the desired operation
 * mode.
 *
 * @param mode will hold the current mode upon successful return. This will be
 *              either WLP_MODE_STA or WLP_MODE_AP.
 *
 * @return
 *   - 0 on success
 *   
 */
int wlp_get_mode(uint8_t *mode);



/**
 * \brief Set the channel. Only supported when mode is WLP_MODE_AP.
 *
 * This function is only supported when wlp_get_mode() returns WLP_MODE_AP.
 * wlp_set_channel() must be called before wlp_linkup() to have effect.
 *
 * @param channel is the new channel (1-13).
 * @param channel_cb Callback that will be invoked when the channel has be
 *              changed successfully. Status will be 0 on success.
 * @param ctx is an opaque context pointer that will be passed back to 
 *              channel_cb.
 *
 * @return
 *   - 0 on success
 *   - WLP_ERR_PARAM if invalid channel
 *   - WLP_ERR_STATE the operation can not be carried out (e.g. ap already
 *              started).
 *   - WLP_ERR_NOTSUPP if operation is not supported
 */
int wlp_set_channel(uint8_t channel,
                    void (*channel_cb)(void *ctx, int status), void *ctx);


/**
 * \brief Configure parameters for access point mode
 *
 * Configure the parameters that will be used when the device is in access
 * point mode, WLP_MODE_AP.
 *
 * @param hidden can be set to non-zero in order to configure the device
 *               with hidden SSID. This means that the device will not
 *               broadcast its SSID, but it will still respond to clients
 *               that already knows the SSID.
 * @param beacon_intvl specifies interval in which beacons are transmitted
 *               by the device. This parameter does not have any effect if
 *               hidden is 0
 * @param inact_tmo specifies the idle time in minutes before the device
 *               will disconnect an inactive client.
 * @param country specifies the region that should be advertised by the device.
 *               Will not be changed if NULL.
 *
 * @return
 *   - 0 on success
 *   - A negative error code on failure
 *
 */
int
wlp_conf_ap(int hidden, int beacon_intvl, int inact_tmo, const char *country);

/*! @} */

/*! \defgroup wlp_ps Functions to manage power consumption
 *
 * These functions manages 802.11 Wi-Fi power save.
 * 
 * @{
 *
 */ 

/** \brief Enable or disable WiFi power save.
 * 
 * Enable/disable legacy power save mode (as opposed to WMM power save
 * which is not supported in the current driver). In legacy power save
 * mode, the device will power down when idle. When connected, the
 * device will wake up to receive beacon frames and any buffered data
 * from the AP. The response time when legacy power save is enabled
 * might therefore be as long as the AP beacon interval (commonly 100
 * ms). However, the throughput should not be affected.
 *
 * @param enable determines if WiFi power save should be enabled or disabled.
 * @return
 *    - 0 on success
 */
int wlp_set_ps(int enable);

/** \brief Configure WiFi power save parameters.
 * 
 * This call is not necessary to enable WiFi power save.
 * If \a wlp_set_ps(1) is called before calling \a wlp_conf_ps()
 * then default power save parameters are used.
 *
 * Beware that setting \a rx_all_dtim to 0 and \a listen_interval to a
 * large value (> 10) can make the device very difficult to
 * reach. When \a rx_all_dtim is 0 you risk loosing multicast data
 * because that data is transferred after a DTIM beacon. This will be
 * particularly noticable when another entitity on the same network
 * segment tries to reach the device and does not know which MAC
 * address the device has. In this case an ARP packet is normally sent
 * to the broadcast address and this will be unlikely to be heard
 * by the device.
 *
 * @param use_ps_poll Use PS-Poll frames to retrieve buffered data.
 *        Any changes to this parameter will take effect upon next
 *        connect or when power save is enabled through
 *        wlp_set_ps().
 *        Note: To retrieve one buffered packet, the ps poll scheme
 *        needs one ps poll packet to the AP instead of two null
 *        packets in the power management bit scheme. Ps poll avoids
 *        the overhead of traffic monitoring time in active mode as
 *        well. But since each ps poll request can make the AP release
 *        only one buffered packet, it is not the optimal scheme for
 *        applications with heavy downlink traffic.
 * @param ps_traffic_timeout Timeout in [ms] to wait for more buffered
 *        data from AP. This setting has no effect if use_ps_poll is
 *        1. Any changes to this parameter will take effect
 *        immediately.
 * @param ps_delay Power save will de delayed ps_delay [ms] after
 *        connecting to an AP.
 * @param rx_all_dtim If set to 1, then STA will wake up to listen to
 *        every beacon containing DTIM (delivery traffic indication
 *        messages) when connected. The actual DTIM interval is
 *        configured in the AP.  If the DTIM interval, as configured
 *        in the AP, is larger than \a listen_interval, the STA will
 *        wakeup according to the \a listen_interval parameter.  Note
 *        that if \a rx_all_dtim is set to 0 and \a listen_interval is
 *        set to something larger than 1 you risk missing reception of
 *        broadcast/multicast data because the AP will transmit that
 *        in conjunction with a DTIM beacon.
 * @param listen_interval The Listen Interval field is used to
 *        indicate to the AP how often a STA in power save mode wakes
 *        to listen to beacon frames. The value of this parameter is
 *        expressed in units of Beacon Interval. An AP may use the
 *        Listen Interval information in determining the lifetime of
 *        frames that it buffers for a STA.  Any changes to this
 *        parameter will take effect upon next association.
 *
 * @return 
 *       - 0 on success. 
 *       - WLP_FAILURE if the configuration failed (most likely
 *         due to invalid parameters).
 */
int wlp_conf_ps(uint8_t use_ps_poll,
                uint32_t ps_traffic_timeout,
                uint32_t ps_delay,
                uint8_t rx_all_dtim,
                uint16_t listen_interval);


/*! @} */

#endif /* WLP_LINK_H */
