/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */


#ifndef WLP_LOOPBACK_H
#define WLP_LOOPBACK_H


/**
 * @file wlp_loopback.h
 *  
 * @author H&D Wireless AB
 * @brief oWL Pico API
 *
 */

#if defined(WLP_CONFIG_TRAMPOLINE)

int _wlp_set_loopback_cb(void (*loop_cb)(void *ctx, void *data, int len),
                         void *ctx);
int _wlp_loopback(void *data, int len);
                  

#else /* defined(WLP_CONFIG_TRAMPOLINE) */

# define wlp_set_loopback_cb    _wlp_set_loopback_cb
# define wlp_loopback           _wlp_loopback

#endif /* defined(WLP_CONFIG_TRAMPOLINE) */


/*! \defgroup wlp_loopback Loopback functions
 *
 * These functions can be used to test the host interface.
 *
 * @{
 *
 */ 

/**
 * \brief Register callback function that will be invoked when receiving
 *      loopback data.
 *
 * @param loop_cb is a pointer to a function that will be invoked when
 *      the device echoes back the data. If NULL, then the device will consume
 *      the data without echoing it.
 * @param ctx is an opaque context pointer that is passed back to loop_cb.
 *
 */
void wlp_set_loopback_cb(void (*loop_cb)(void *ctx, void *data, int len),
                         void *ctx);


/**
 * \brief Send data buffer to device.
 *
 * @param data is the buffer to send
 * @param len is the length of the buffer
 *
 * @return
 *   - 0 on success.
 *
 */
int wlp_loopback(void *data, int len);

    
/*! @} */

#endif /* WLP_LOOPBACK_H */
