/*
 * Copyright (C) 2012, H&D Wireless AB All rights reserved.
 *
 * The license to use this software in whole and in part and to
 * redistribute it in any form follows with the WiFi HW module from H&D
 * Wireless and is granted under the following restrictions:
 * 
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 *
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * 3. The name of H&D Wireless AB may not be used to endorse or promote
 *    products derived from this software without specific prior written
 *    permission.
 *
 * 4. The software may only be used together with hardware from H&D
 *    Wireless all other use is prohibited.
 *
 * 5. The license to use and redistribute the software is granted
 *    together with the purchase of a hardware platform on a one to one
 *    basis.
 *
 * 6. The binary code may not be reversed engineered or by other means
 *    copied to circumvent this license.
 * 
 * THIS SOFTWARE IS PROVIDED BY H&D WIRELESS AB ``AS IS'' AND ANY
 * EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE EXPRESSLY AND SPECIFICALLY DISCLAIMED. IN NO EVENT
 * SHALL HD WIRELESS AB BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 * (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * For more information regarding this software license Contact H&D
 * Wireless AB (support@hd-wireless.se).
 *
 */


#ifndef WLP_SERVICE_H
#define WLP_SERVICE_H

#include <wlp_ipaddr.h>


/**
 * @file wlp_service.h
 *  
 * @author H&D Wireless AB
 * @brief oWL Pico API
 *
 */

#if defined(WLP_CONFIG_TRAMPOLINE)

int _wlp_set_dhcpd(int enable);

#else /* defined(WLP_CONFIG_TRAMPOLINE) */

#define wlp_set_dhcpd           _wlp_set_dhcpd

#endif /* defined(WLP_CONFIG_TRAMPOLINE) */


/*! \defgroup wlp_service Network service functions
 *
 * These functions manage various network services.
 *
 * @{
 *
 */ 

/**
 * \brief Start or stop the device DHCP server.
 *
 * Enable or disable DHCP server for IP address configuration.
 *
 * wlp_set_dhcpd() can be invoked either before or after the wifi link is up.
 * If this function is called when the wifi link is down, any DHCP services
 * will be performed when the wifi link comes up.
 *
 * Technically, it is possible to enable the DHCP server in STA mode or AP
 * mode. However, it seldom makes sense to use a DHCP server in STA mode.
 *
 * The DHCP server will offer IP addresses in the range x.y.z.100 to x.y.z.255.
 * For instance; if the ip address of the device is 192.168.1.1 and
 * netmask 255.255.255.0 is used, the DHCP server will offer IP addresses in
 * the range 192.168.1.100 to 192.168.1.255. If the netmask was 255.255.0.0 the
 * range would be the same. In case the netmask is something like 255.255.255.y,
 * where y != 0, we would mask out the some of the IP addresses in the above
 * range to make sure that all offered IP addresses are actually valid on the
 * given network.
 *
 * When using the DHCP server, a static IP address must be assigned to
 * the device. The gateway address that is set with \a
 * wlp_set_ipaddr() will be used as the address in the "router" DHCP
 * option. Some systems (notably IOS devices) will not bring the link
 * up completely unless this option is a valid address so it is
 * usually wise to set the gateway address to the same static IP
 * address that is used for the DHCP Server itself (unless you know
 * that it should be set to something particular).
 *
 * @param enable determines if the DHCP server should be enabled or disabled.
 *
 * @return
 *   - 0 on success.
 *
 */
int wlp_set_dhcpd(int enable);

    
/*! @} */

#endif /* WLP_SERVICE_H */
